/* COMMAND.C -- command decoder                                              */



/*---------------------------------------------------------------------------*/
/* Create network address from P1 bits                                       */
/* The next-to-last character of the string gets the P1 digit                */
/*  (the string should always end in a blank, but we do not check for this)  */

#ifdef NETWORK

int CmdMakeAddress() {

BYTE AddrBits;
BYTE AddrChar;
BYTE AddrLen;

   AddrBits = ((~P1) & 0x0E) >> 1;     /* get input bits, flip, & align      */
   AddrChar = AddrBits | '0';          /* turn into character                */

   AddrLen = strlen(CmdNetAddress);    /* insert it in the string            */
   if (2 <= AddrLen) {
      CmdNetAddress[AddrLen - 2] = AddrChar;
   }
   else {
      CmdNetAddress[0] = AddrChar;
   }

   return (int)AddrBits;

}

#endif


/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */

#ifdef NETWORK

int CmdCheckMode() {

asm {
   SETB  MANUALMODE                    ; ensure input on bit
   MOV   C,MANUALMODE                  ; pick up manual mode input
   CPL   C                             ; flip so jumper installed = 1
   CLR   A                             ; start with zero
   MOV   A.0,C                         ; combine with control bit
   MOV   TestBits,A                    ; set up for C code
}

   if (TestBits) {
      CmdNetMode(0);                   /* jumper installed sets manual mode  */
   }
   else {
      CmdNetMode(NETMODE);             /* all other addresses are automatic  */
   }

   return;

}

#endif


/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
WORD DevCode;
WORD PortValue;

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'C' :                          /* hardware configuration             */
   /* CmdShowMsg("*** CONFIG command not ready yet...\n"); */
      break;
   case 'D' :                          /* dump internal or stored buffers    */
      CmdDecDumpState();
      sprintf(CmdRespBuff,"T0Ticks %u\n",T0Ticks);
      Nputstr(CmdRespBuff);
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      break;
   case 'L' :                          /* set debug logging options          */
      CmdDecLogging(pCmd);
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'Q' :                          /* query input values                 */
      if (('A' == *pChar) && ('1' >= *(pChar+1))) {         /* QA commands?  */
         ++pChar;                                  /* step to board number   */
         switch (*pChar) {
         case 0 :
            IODumpAll(0);
            Nputch('\n');
            IODumpAll(1);
            break;
         case '0' :
         case '1' :
            IODumpAll(*pChar - '0');
            break;
         default :
            CmdShowMsg("*** Invalid QA command");
         }
      }
      else {
         while (*pChar) {
            if (IOGetValue(&pChar,&DevCode,&PortValue)) {
               break;
            }
            switch (IO_DEVICE(DevCode)) {
            case DEV_CLK0 :
            case DEV_CLK1 :
               if (NOCHANSPEC & DevCode) {
                  IOFormatTime(DevCode,IOTimeFormat);   /* special case      */
               }
               else {
                  IOFormatData(DevCode,PortValue);
               }
               break;
            default :
               IOFormatData(DevCode,PortValue);
            }
            if (',' == *pChar) {          /* step over separator                */
               ++pChar;
               Nputch(',');               /* add one to output...               */
            }
         }
      }
      Nputch('\n');
      break;
   case 'R' :
      if (strcmp(pChar,"ESET")) {                  /* nonzero = no match     */
         CmdShowMsg("*** Unknown command\n");
      }
      else {
         SerWaitOutDone();
 asm {
   CLR   EA
   LJMP   0                            ; solid reset...
 }
      }
      break;
   case 'S' :                          /* set output value                   */
      while (*pChar) {
         if (IOSetValue(&pChar,&DevCode)) {        /* handles clk strings    */
            break;
         }
         if (',' == *pChar) {          /* step over separator                */
            ++pChar;
         }
      }
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      Nputstr("Available commands (values in decimal unless noted)\n");
      Nputstr(" A=string   + set network address to string\n");
/*    Nputstr(" CONFIG     * set hardware/firmware configuration\n"); */
      Nputstr(" D          * display internal status variables (some in hex)\n");
      Nputstr(" E          * show and clear error flags\n");
      Nputstr(" L n          set reporting mode (bitmapped)\n");
      Nputstr(" N n       *+ set network mode\n");
      Nputstr(" Q            query inputs\n");
      Nputstr(" RESET        perform power-on reset initialization\n");
      Nputstr(" S            set output values\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;

}

