/* COMMAND.C -- command decoder                                              */



/*---------------------------------------------------------------------------*/
/* Create network address from switch inputs                                 */
/* This is a no-op because we have no switches on COMM-Link Version 2 boards */

#ifdef NETWORK

int CmdMakeAddress() {

   return 0;

}

#endif


/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */

#ifdef NETWORK

int CmdCheckMode() {

asm {
   SETB  DIS_INPORT                    ; disable input bits
   SETB  STB_OUTPORT                   ; ensure we are not latching
   SETB  DIS_ADC                       ; disable ADC outputs

   SETB  MANUALMODE                    ; allow input on bit
   CLR   DIS_EEPROM                    ; enable input from manual mode

   MOV   B,#50                         ; allow settling time!
?man1
   DJNZ  B,?man1

   MOV   C,MANUALMODE                  ; pick up the bit
   CPL   C                             ; jumper installed = 1
   CLR   A                             ; flush the variable
   MOV   A.0,C                         ; set up the bit
   MOV   TestBits,A                    ; set up for C code
}

   if (TestBits) {
      CmdNetMode(0);                   /* we are in manual mode!             */
   }
   else {
      CmdNetMode(NETMODE);             /* all other addresses are automatic  */
   }

   return;

}

#endif



/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
WORD DevCode;
WORD PortValue;
BYTE Temp;

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'C' :                          /* hardware configuration             */
      if (*pChar) {
         Temp = 0;
         while (ADCNames[Temp] &&
                strcmp(pChar,ADCNames[Temp])) {
            ++Temp;
         }
         if (NULL == ADCNames[Temp]) {
            Nputstr("*** Invalid ADC name: [");
            Nputstr(pChar);
            Nputstr("]\n");
         }
         else {
            IOConfigADC(Temp);
            CfgPutConfig();
         }
      }
      else {
         IOShowConfig();
      }
      break;
   case 'D' :                          /* dump internal or stored buffers    */
      CmdDecDumpState();
      sprintf(CmdRespBuff,"T0Ticks %u\n",T0Ticks);
      Nputstr(CmdRespBuff);
      CfgDumpState();
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      break;
   case 'L' :                          /* set debug logging options          */
      CmdDecLogging(pCmd);
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'Q' :                          /* query input values                 */
      while (*pChar) {
         if (IOGetValue(&pChar,&DevCode,&PortValue)) {
            break;
         }
         switch (IO_DEVICE(DevCode)) {
         default :
            IOFormatData(DevCode,PortValue);
         }
         if (',' == *pChar) {          /* step over separator                */
            ++pChar;
            Nputch(',');               /* add one to output...               */
         }
         TmrHeartBeat();
      }
      Nputch('\n');
      break;
   case 'R' :
      if (strcmp(pChar,"ESET")) {                  /* nonzero = no match     */
         CmdShowMsg("*** Unknown command\n");
      }
      else {
         SerWaitOutDone();
 asm {
   CLR   EA                            ; ensure no interruptions
   LJMP   0                            ; solid reset...
 }
      }
      break;
   case 'S' :                          /* set output value                   */
      while (*pChar) {
         if (IOSetValue(&pChar,&DevCode)) {        /* handles clk strings    */
            break;
         }
         if (',' == *pChar) {          /* step over separator                */
            ++pChar;
         }
         TmrHeartBeat();
      }
      break;

   case 'X' :                          /* clobber EEPROM contents            */
      CfgSendEECmd(EEP_EWEN,0);
      CfgSendEECmd(EEP_ERASE,0);
      CfgSendEECmd(EEP_EWDS,0);
      CmdShowMsg("EEPROM contents erased\n");
      CmdShowMsg(" Use RESET command to write default configuration\n");
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      TmrHeartBeat();                              /* check for heartbeat    */
      Nputstr("Available commands (values in decimal unless noted)\n");
      Nputstr(" A=string   + set network address to string\n");
      Nputstr(" C          * set hardware/firmware configuration\n");
      Nputstr(" D          * display internal status variables (some in hex)\n");
      TmrHeartBeat();                              /* check for heartbeat    */
      Nputstr(" E          * show and clear error flags\n");
      Nputstr(" L n          set reporting mode (bitmapped)\n");
      Nputstr(" N n       *+ set network mode\n");
      Nputstr(" Q            query inputs\n");
      TmrHeartBeat();                              /* check for heartbeat    */
      Nputstr(" RESET        perform power-on reset initialization\n");
      Nputstr(" S            set output values\n");
      Nputstr(" X            erase EEPROM contents (wipe configuration)\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      TmrHeartBeat();                              /* check for heartbeat    */
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;

}

