/* IODEV.C                                                                   */

/*---------------------------------------------------------------------------*/
/* Set up I/O devices and initialize the arrays                              */

IOInitialize() {

/*--- port values                                                            */

   SETBIT(DIS_ADC);
   SETBIT(DIS_INPORT);
   SETBIT(STB_OUTPORT);
   SETBIT(DIS_EEPROM);

/*--- global variables                                                       */

   CmdShowMsg("Initializing device information\n");

   memset(IOFlags,0,sizeof(IOFlags));
   memset(IOCurrent,0,sizeof(IOCurrent));
   memset(IONumChan,0,sizeof(IONumChan));
   memset(IONumDrv,0,sizeof(IONumDrv));

/*--- set up device control variables                                        */

   if (EEPStatus == EES_VALID) {
      IOGetEEImage();

   }
   else {
      CmdShowMsg("  Setting default configuration\n");

/*    AI is assumed to be the simplest chip                                  */

      IOConfigADC(ADC_ADC0831);

/*    DI is required for LED output and manual mode input                    */

      IOFlags[DEV_DI][0] = IO_INSTALLED;
      IONumChan[DEV_DI] = 1;
      IONumDrv[DEV_DI][0] = 4;

/*    DO may not be installed, but we cannot tell from in here...            */

      IOFlags[DEV_DO][0] = IO_WRITEONLY | IO_INSTALLED;
      IONumChan[DEV_DO] = 1;
      IONumDrv[DEV_DO][0] = 4;


      IOPutEEImage();

   }

/*--- set up outputs to known default state                                  */

   IOWriteDev(DEV_DO,0,0);

/*--- report on ADC setup                                                    */

   IOShowConfig(1);

}



/*---------------------------------------------------------------------------*/
/* Show I/O configuration settings                                           */

IOShowConfig() {

BYTE Temp;

   Nputstr("Configuration:\n");
   Temp = IONumDrv[DEV_AI][0];
   sprintf(CmdRespBuff,"  ADC type %u = %s\n",Temp,ADCNames[Temp]);
   Nputstr(CmdRespBuff);

}

/*---------------------------------------------------------------------------*/
/* Transfer EEPROM image to our variables                                    */

IOGetEEImage() {

   move(IOFlags,  EEImage+2*(EE_IOSETUP),
        sizeof(IOFlags));
   move(IONumChan,EEImage+2*(EE_IOSETUP+sizeof(IOFlags)),
        sizeof(IONumChan));
   move(IONumDrv, EEImage+2*(EE_IOSETUP+sizeof(IOFlags)+sizeof(IONumChan)),
        sizeof(IONumDrv));

}

/*---------------------------------------------------------------------------*/
/* Transfer our variables to EEPROM image                                    */

IOPutEEImage() {

   move(EEImage+2*(EE_IOSETUP),
        IOFlags,sizeof(IOFlags));
   move(EEImage+2*(EE_IOSETUP+sizeof(IOFlags)),
        IONumChan,sizeof(IONumChan));
   move(EEImage+2*(EE_IOSETUP+sizeof(IOFlags)+sizeof(IONumChan)),
        IONumDrv,sizeof(IONumDrv));

}

/*---------------------------------------------------------------------------*/
/* Set up ADC configuration parameters                                       */
/* Does not write them to the EEPROM                                         */

IOConfigADC(ADCType)
WORD ADCType;
{

   IOFlags[DEV_AI][0] = IO_WORD | IO_CHANNELS | IO_INSTALLED;
   IONumDrv[DEV_AI][0] = ADCType;

   switch (ADCType) {
   case ADC_NONE :
      IOFlags[DEV_AI][0] = 0;          /* removes the current value          */
      break;
   case ADC_ADC0831 :                  /* no output, so no selections...     */
   case ADC_LTC1092 :
   case ADC_LTC1292 :
      IONumChan[DEV_AI] = 1;
      break;
   case ADC_ADC0832 :
      IONumChan[DEV_AI] = 4;           /* select single/diff + polarity      */
      break;
   }

   IOPutEEImage();                     /* transfer to EEPROM image area      */

}

/*---------------------------------------------------------------------------*/
/* Write output data to device                                               */
/* If it is not readable, save the value for simulated readbacks             */

IOWriteDev(DevNdx,DevOffs,Value)
WORD DevNdx;
WORD DevOffs;
WORD Value;
{

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Write %04x to device ID %04x\n",
              Value,DevNdx);
      CmdShowMsg(CmdRespBuff);
   }

   switch (DevNdx) {
   case DEV_AI :                       /* this is not writable               */
   case DEV_DI :                       /* this is not writable               */
      break;
   case DEV_DO :
      Value &= DATAMASK;               /* strip off junk                     */
      SETBIT(DIS_EEPROM);              /* disable manual input & heartbeat   */
asm {
      MOV R0,#-5                       ; point to Value on stack
      LCALL ?auto0
      MOV   A,[R0]                     ; fetch low byte
      MOV   B,A                        ; save for later

      MOV   A,DATAPORT                 ; fetch strobes and junk
      ANL   A,#~DATAMASK               ; strip off junk
      ORL   A,B                        ; tack on new output bits
      MOV   DATAPORT,A                 ; send to driver input

      CLR   STB_OUTPORT                ; latch into driver chip
      SETB  STB_OUTPORT

      ORL   A,#DATAMASK                ; disable outputs
      MOV   DATAPORT,A                 ;  ... just for nice

}
      TmrHeartBeat();                  /* restart heartbeat                  */
      break;
   }

   if (IO_WRITEONLY & IOFlags[DevNdx][DevOffs]) {
      IOCurrent[DevNdx][DevOffs] = Value;
   }

}


/*---------------------------------------------------------------------------*/
/* Convert ADC input                                                         */
/* Returns the converted value, FFFF if there is a problem we can detect     */

WORD IOConvADC(DevNdx,DevChan,Ctl)
WORD DevNdx;
WORD DevChan;
int Ctl;
{
WORD RetVal;

   DevChan;                            /* otherwise unused                   */
   Ctl;

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Convert dev %u, chan %u, type %u\n",
              DevNdx,DevChan,IONumDrv[DevNdx][0]);
      CmdShowMsg(CmdRespBuff);
   }

   RetVal = 0;

   switch (IONumDrv[DevNdx][0]) {
   case ADC_NONE :                     /* no ADC configured                  */
      TestBits = 1;                    /* force error return value           */
      break;

   case ADC_ADC0831:
      asm {
         MOV   R7,#0                   ; aim at RetVal LOW byte
         LCALL ?accex1

         SETB  ADC_SIN                 ; ensure high if mis-jumpered
         SETB  ADC_SOUT                ; allow read from ADC
         CLR   ADC_CLK                 ; clock starts low
         CLR   DIS_ADC                 ; enable converter

         MOV   B,#10                   ; start, dummy, 8 high-order data bits
         CLR   EA                      ; ensure good timings

?ADC31
         SETB  ADC_CLK                 ; restore clock in middle of bit
         MOV   C,ADC_SOUT              ; pick up input bit
         RLC   A                       ; tack onto accumulator
         CLR   ADC_CLK                 ; pump out next bit
         DJNZ  B,?ADC31                ; repeat until done

         SETB  EA

         MOVX  [DPTR],A                ; save byte

         CLR   A                       ; error if dummy bit was 1
         MOV   A.0,C
         MOV   TestBits,A

         SETB  DIS_ADC                 ; disable converter
         SETB  ADC_CLK                 ; float the clock
      }
      break;

   case ADC_ADC0832:
      asm {
         MOV   R7,#0                   ; aim at RetVal LOW byte
         LCALL ?accex1

         MOV   R0,#-7                  ; aim at DevChan LOW byte
         LCALL ?auto0
         MOV   A,[R0]
         ORL   A,#$04                  ; start bit, two bits from channel ID
         SWAP  A                       ; align in MSB
         RLC   A

         SETB  ADC_SIN                 ; ensure high if mis-jumpered
         SETB  ADC_SOUT                ; allow read from ADC
         CLR   ADC_CLK                 ; clock starts low
         CLR   DIS_ADC                 ; enable converter

         MOV   B,#3                    ; shift out three bits
         CLR   EA                      ; ensure good timings

?ADC32o
         CLR   ADC_CLK                 ; set clock inactive
         RLC   A                       ; snag bit
         MOV   ADC_SIN,C               ; send to ADC
         SETB  ADC_CLK                 ; latch the bit in the ADC
         DJNZ  B,?ADC32o               ; repeat until done

         CLR   ADC_CLK                 ; set clock inactive

         MOV   B,#9                    ; dummy, 8 high-order data bits

?ADC32i
         SETB  ADC_CLK                 ; restore clock in middle of bit
         MOV   C,ADC_SOUT              ; pick up input bit
         RLC   A                       ; tack onto accumulator
         CLR   ADC_CLK                 ; pump out next bit
         DJNZ  B,?ADC32i               ; repeat until done

         SETB  EA

         MOVX  [DPTR],A                ; save byte

         CLR   A                       ; error if dummy bit was 1
         MOV   A.0,C
         MOV   TestBits,A

         SETB  DIS_ADC                 ; disable converter
         SETB  ADC_SIN                 ; float data to ADC
         SETB  ADC_CLK                 ; float the clock
      }
      break;

   case ADC_LTC1092:
      asm {
         MOV   R7,#0                   ; aim at RetVal LOW byte
         LCALL ?accex1

         SETB  ADC_SIN                 ; ensure high if mis-jumpered
         SETB  ADC_SOUT                ; allow read from ADC
         CLR   ADC_CLK                 ; clock starts low
         CLR   DIS_ADC                 ; enable converter

         MOV   B,#10                   ; start, dummy, 8 high-order data bits
         CLR   EA

?LTC10H
         SETB  ADC_CLK                 ; restore clock in middle of bit
         MOV   C,ADC_SOUT              ; pick up input bit
         RLC   A                       ; tack onto accumulator
         CLR   ADC_CLK                 ; pump out next bit
         DJNZ  B,?LTC10H               ; repeat until done

         MOV   R1,A                    ; save high byte
         MOV   F0FLAG,C                ; save dummy bit

         MOV   B,#2                    ; remaining bits
         CLR   A                       ; with zero fill

?LTC10L
         SETB  ADC_CLK                 ; restore clock in middle of bit
         MOV   C,ADC_SOUT              ; pick up input bit
         RLC   A                       ; tack onto accumulator
         CLR   ADC_CLK                 ; pump out next bit
         DJNZ  B,?LTC10L               ; repeat until done

         SETB  EA

         MOV   B,#64                   ; shift 6 bits leftward
         MUL   AB

         MOVX  [DPTR],A                ; save low byte
         INC   DPTR                    ; aim at high byte
         MOV   A,R1
         MOVX  [DPTR],A                ; save high byte

         CLR   A                       ; error if dummy bit was 1
         MOV   C,F0FLAG
         MOV   A.0,C
         MOV   TestBits,A

         SETB  DIS_ADC                 ; disable converter
         SETB  ADC_CLK                 ; float the clock
      }
      RetVal >>= 6;                    /* align the results                  */
      break;

   case ADC_LTC1292:
      asm {
         MOV   R7,#0                   ; aim at RetVal LOW byte
         LCALL ?accex1

         SETB  ADC_SIN                 ; ensure high if mis-jumpered
         SETB  ADC_SOUT                ; allow read from ADC
         CLR   ADC_CLK                 ; clock starts low
         CLR   DIS_ADC                 ; enable converter

         MOV   B,#10                   ; start, dummy, 8 high-order data bits
         CLR   EA

?LTC12H
         SETB  ADC_CLK                 ; restore clock in middle of bit
         MOV   C,ADC_SOUT              ; pick up input bit
         RLC   A                       ; tack onto accumulator
         CLR   ADC_CLK                 ; pump out next bit
         DJNZ  B,?LTC12H               ; repeat until done

         MOV   R1,A                    ; save high byte
         MOV   F0FLAG,C                ; save dummy bit

         MOV   B,#4                    ; remaining bits
         CLR   A                       ; with zero fill

?LTC12L
         SETB  ADC_CLK                 ; restore clock in middle of bit
         MOV   C,ADC_SOUT              ; pick up input bit
         RLC   A                       ; tack onto accumulator
         CLR   ADC_CLK                 ; pump out next bit
         DJNZ  B,?LTC12L               ; repeat until done

         SETB  EA

         SWAP  A                       ; align in high bits

         MOVX  [DPTR],A                ; save low byte
         INC   DPTR                    ; aim at high byte
         MOV   A,R1
         MOVX  [DPTR],A                ; save high byte

         CLR   A                       ; error if dummy bit was 1
         MOV   C,F0FLAG
         MOV   A.0,C
         MOV   TestBits,A

         SETB  DIS_ADC                 ; disable converter
         SETB  ADC_CLK                 ; float the clock
      }
      RetVal >>= 4;                    /* align the results                  */
      break;
   }

   if (TestBits) {
      RetVal = 0xFFFF;
      ErrFlags |= ERR_ADC;
      CmdShowMsg("*** ADC hardware failure, no leading 0 bit\n");
   }

   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Read data from device                                                     */
/* If it is not readable, we use the value stored in the IOCurrent array     */

WORD IOReadDev(DevNdx,DevOffs)
WORD DevNdx;
WORD DevOffs;
{
WORD RetValue;
BYTE Faked;


   if (IO_WRITEONLY & IOFlags[DevNdx][DevOffs]) {
      Faked = 1;
      RetValue = IOCurrent[DevNdx][DevOffs];
   }
   else {
      Faked = 0;
      switch (DevNdx) {
      case DEV_AI :
         RetValue = IOConvADC(DevNdx,DevOffs,0);
         break;
      case DEV_DI :
         SETBIT(DIS_EEPROM);           /* turn off manual mode input         */
         CLRBIT(DIS_INPORT);           /* enable input port drivers          */
         RetValue = DATAPORT & DATAMASK;           /* get data, strip junk   */
         RetValue &= 0x00FF;                       /* strip off high byte    */
         SETBIT(DIS_INPORT);           /* turn off drivers                   */
         break;
      case DEV_DO:                     /* this will not happen               */
         break;
      }
   }

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Read %04x from device ID %04x %s\n",
              RetValue,DevNdx,
              Faked ? "faked" : "");
      Nputstr(CmdRespBuff);
   }

   return RetValue;

}



/*---------------------------------------------------------------------------*/
/* Parse I/O device specifier into array index, channel offset, and bit ID   */
/* Argument is pointer to pointer to characters                              */
/* Returns index in high byte, channel in low byte                           */
/* Returns NODEVSPEC if device is no good                                    */
/* Always returns char ptr incremented past last char used                   */

int IOParseDev(ppSpec)
char **ppSpec;
{
WORD RetVal;
char *pBuffer;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
BYTE Temp;

   DevChan = DevBitID = 0;
   RetVal = NODEVSPEC | NOCHANSPEC | NOBITSPEC;    /* assume the worst       */

/*--- extract device string up to first non-alphanumeric character           */
/*    then look it up in the table for a match                               */

   memset(IOParseBuff,0,sizeof(IOParseBuff));
   pBuffer = IOParseBuff;
   while (isascii(**ppSpec) && isalnum(**ppSpec)) {         /* force caps    */
      *(pBuffer++) = toupper(**ppSpec);
      ++(*ppSpec);
   }

   for (DevNdx=0; DevNdx<NUMDEVS; ++DevNdx) {
      if (!strcmp(IOParseBuff,IOName[DevNdx])) {
         break;
      }
   }

   if (NUMDEVS == DevNdx) {
      sprintf(CmdRespBuff,"*** Bad device name [%s]\n",IOParseBuff);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }
   else {
      RetVal &= ~NODEVSPEC;
   }

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"[Device %s ID %02x",IOParseBuff,DevNdx);
      Nputstr(CmdRespBuff);
   }

/*--- extract channel spec if we ended at a dot                              */
/*    if there are no channels, convert number into bit spec                 */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (IO_CHANNELS & IOFlags[DevNdx][0]) {
         if (Temp < IONumChan[DevNdx]) {
            DevChan = Temp;
            RetVal &= ~NOCHANSPEC;
            if (REPORT_DEV & ReportMode) {
               sprintf(CmdRespBuff,", channel = %u",Temp);
               Nputstr(CmdRespBuff);
            }
         }
         else {
            RetVal = NODEVSPEC;
            sprintf(CmdRespBuff,"\n*** Bad channel for %s: %u\n",
                     IOName[DevNdx],Temp);
            CmdShowMsg(CmdRespBuff);
            goto Done;
         }
      }
      else {
         if (Temp < IONumDrv[DevNdx][0]) {
            DevChan = 0;
            DevBitID = Temp;
            RetVal &= ~(NOCHANSPEC | NOBITSPEC);
            if (REPORT_DEV & ReportMode) {
               sprintf(CmdRespBuff,", driver = %u",Temp);
               Nputstr(CmdRespBuff);
            }
         }
         else {
            RetVal = NODEVSPEC;
            sprintf(CmdRespBuff,"\n*** Bad driver bit for %s: %u\n",
                     IOName[DevNdx],Temp);
            CmdShowMsg(CmdRespBuff);
            goto Done;
         }
      }
   }
   else {
      if (REPORT_DEV & ReportMode) {
         Nputstr(", all channels/drivers");
      }
   }


/*--- extract bit spec if we are once again at a dot                         */
/*    if we have no channels, use the channel spec as a bit selector         */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (((Temp < 16) && (IO_WORD & IOFlags[DevNdx][0])) ||
          ((Temp <  8) && !(IO_WORD & IOFlags[DevNdx][0]))) {
         RetVal &= ~NOBITSPEC;
         DevBitID = Temp;
         if (REPORT_DEV & ReportMode) {
            sprintf(CmdRespBuff,", bit ID = %u]\n",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         RetVal = NODEVSPEC;
         sprintf(CmdRespBuff,"\n*** Bad bit ID: %u\n",Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEV & ReportMode) {
         Nputstr("]\n");
      }
   }


/*--- combine the parts and return a single word                             */
/*    preserves all the flags in the high nibble                             */
/*     (values will be zero if flags are set, but so it goes                 */

Done:

   RetVal |= (DevNdx << 8) | (DevChan << 4) | DevBitID;

   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Extract I/O device spec and value from string, set the output hardware    */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Extracts device index/channel/bit and returns the code                    */
/* Function return value is 0 for OK, 1 for error                            */

int IOSetValue(ppCmd,pDevCode)
char **ppCmd;
WORD *pDevCode;
{
int RetCode;
WORD DevCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
WORD Value;
WORD PortVal;

   RetCode = 1;                                    /* assume failure         */

   *pDevCode = DevCode = IOParseDev(ppCmd);        /* get device & channel   */
   if (NODEVSPEC & DevCode) {                      /* skip invalid devs      */
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   if (('=' != **ppCmd) || !(*((*ppCmd)+1))) {     /* check for =string      */
      CmdShowMsg("*** Set command missing data\n");
      goto Done;
   }

   ++(*ppCmd);                                     /* step over = sign       */

   if (NOCHANSPEC & DevCode) {                     /* missing channel num?   */
      if (IONumChan[DevNdx] > 1) {                 /* do we need it?         */
         CmdShowMsg("*** Missing channel\n");
         goto Done;
      }
      else {
         DevChan = 0;                              /* no, force first chan   */
      }
   }

   Value = GetHexits(ppCmd,4);                     /* get hex word if there  */

   if (NOBITSPEC & DevCode) {                      /* missing bit spec?      */
      IOWriteDev(DevNdx,DevChan,Value);            /* write whole byte       */
   }
   else {                                          /* have bit map spec      */
      if (Value > 1)    {                          /* is data OK?            */
         CmdShowMsg("*** Bit value must be 0 or 1\n");
         goto Done;
      }
      PortVal = IOReadDev(DevNdx,DevChan);         /* fetch existing data    */
      PortVal &= ~(0x01 << DevBitID);              /* remove old bit         */
      PortVal |= Value << DevBitID;                /* insert new bit         */
      IOWriteDev(DevNdx,DevChan,PortVal);          /* write whole byte       */
   }
   RetCode = 0;


Done:

   return RetCode;

}



/*---------------------------------------------------------------------------*/
/* Extract I/O device from incoming string and return current value          */
/* Returns value from appropriate port or bit                                */
/* Function return value is 0 for OK, 1 for error                            */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Returns decoded value of device and channel                               */
/* If the device is not installed, displays error message                    */

int IOGetValue(ppCmd,pDevCode,pValue)
char **ppCmd;
WORD *pDevCode;
WORD *pValue;
{

int RetCode;
WORD DevCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
WORD Value;

   *pDevCode = DevCode = IOParseDev(ppCmd);        /* get device & channel   */
   RetCode = 1;                                    /* assume error           */
   Value = 0;                                      /* just in case...        */

   if (NODEVSPEC & DevCode) {                      /* bad device             */
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   if (!(IO_INSTALLED & IOFlags[DevNdx][0])) {              /* installed?    */
      sprintf(CmdRespBuff,"*** Device %s not installed\n",IOName[DevNdx]);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }

   if ('=' == **ppCmd) {                           /* check for = sign!      */
      CmdShowMsg("*** Cannot set value\n");
      goto Done;
   }
   else {

      if (NOCHANSPEC & DevCode) {                  /* missing channel?       */
         DevChan = 0;                              /* default to zero here   */
      }

      if (NOBITSPEC & DevCode) {
         Value = IOReadDev(DevNdx,DevChan);
      }
      else {                                       /* use bit spec on data   */
         Value = IOReadDev(DevNdx,DevChan);        /* fetch existing data    */
         Value >>= DevBitID;                       /* move bit to LSB        */
         Value &= 0x0001;                          /* isolate it             */
      }
   }


   RetCode = 0;

Done:

   *pValue = Value;

   return RetCode;

}



/*---------------------------------------------------------------------------*/
/* Formats value for given device and channel                                */
/* Produces appropriate text to identify the results                         */
/* Message for non-installed devices                                         */

int IOFormatData(DevCode,Value)
WORD DevCode;
WORD Value;
{
int RetCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
char *pFormat;

   RetCode = 1;

   if (NODEVSPEC & DevCode) {
      strcpy(CmdRespBuff,"*** Bad device\n");
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   strcpy(CmdRespBuff,IOName[DevNdx]);             /* set up device name     */

   if (!(NOCHANSPEC & DevCode)) {
      sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevChan);
      if (!(NOBITSPEC & DevCode)) {
         sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevBitID);
      }
   }

   strcat(CmdRespBuff,"=");

   if (!(NOBITSPEC & DevCode)) {                   /* have bit specifier     */
      pFormat = "%u";                              /* either 0 or 1          */
   }
   else {
      pFormat = (IO_WORD & IOFlags[DevNdx][0]) ? "%04x" : "%02x";
   }

   sprintf(CmdRespBuff+strlen(CmdRespBuff),pFormat,Value);

   RetCode = 0;

Done:

   if (RetCode) {
      CmdShowMsg(CmdRespBuff);                     /* suppressable errors    */
   }
   else {
      Nputstr(CmdRespBuff);                        /* normal results         */
   }

   return RetCode;

}

