/* IODEV.C                                                                   */

#define TRC_IOxxx
#define TRC_ESCxxx


/*---------------------------------------------------------------------------*/
/* Write output data to port                                                 */
/* If port is not readable, save the value for simulated readbacks           */
/* Special bypass for Port 1 and Port 3 data...                              */
/*  also update input bit masks on each output to prevent input latching     */
/* Blips strobe bit after each update                                        */

IOWritePort(DevNdx,DevOffs,Value)
WORD DevNdx;
WORD DevOffs;
WORD Value;
{

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Write %02x to address %04x\n",
              Value,IOAddress[DevNdx]+DevOffs);
      CmdShowMsg(CmdRespBuff);
   }

   if (IO_CPUPORT & IOFlags[DevNdx][0]) {
      switch (IOAddress[DevNdx]) {
      case 0x0001 :
         IOInBits[DevNdx][DevOffs] &= Value;       /* any zeros are not INs  */
         Value |= IOInBits[DevNdx][DevOffs];       /* any INs are now ones   */
         P1 = Value;
         break;
      case 0x0003 :
         IOInBits[DevNdx][DevOffs] &= Value;       /* any zeros are not INs  */
         Value |= IOInBits[DevNdx][DevOffs];       /* any INs are now ones   */
         P3 = Value;
         break;
      }
   }
   else {
      poke(IOAddress[DevNdx]+DevOffs,Value);
   }

/*--- blip the strobe                                                        */

   CLRBIT(STROBEBIT);
   TmrDelay100US(2);
   SETBIT(STROBEBIT);

/*--- update stored values if this is a write-only port...                   */
/*    all others will read the current value and update the IOCurrent array  */

   if (IO_WRITEONLY & IOFlags[DevNdx][DevOffs]) {
      IOCurrent[DevNdx][DevOffs] = Value;
   }


}


/*---------------------------------------------------------------------------*/
/* Read data from port                                                       */
/* If port is not readable, we use the value stored in the IOCurrent array   */
/* CPU port bits are not affected by input mask bits here...                 */

WORD IOReadPort(DevNdx,DevOffs)
WORD DevNdx;
WORD DevOffs;
{
WORD RetValue;
BYTE Faked;


   if (IO_WRITEONLY & IOFlags[DevNdx][DevOffs]) {  /* simulate the inputs    */
      RetValue = IOCurrent[DevNdx][DevOffs];
      Faked = 1;
   }
   else {                                          /* use real values        */
      Faked = 0;
      if (IO_CPUPORT & IOFlags[DevNdx][0]) {
         switch (IOAddress[DevNdx]) {
         case 0x0001 :
            RetValue = (WORD)P1;
            break;
         case 0x0003 :
            RetValue = (WORD)P3;
            break;
         }
      }
      else {
         RetValue = peek(IOAddress[DevNdx]+DevOffs);
      }
   }

   if (REPORT_DEV & ReportMode) {
      sprintf(CmdRespBuff,"Read %02x from address %04x %s\n",
              RetValue,IOAddress[DevNdx]+DevOffs,
              Faked ? "faked" : "");
      Nputstr(CmdRespBuff);
   }

   return RetValue;

}


/*---------------------------------------------------------------------------*/
/* Set up I/O devices and initialize the arrays                              */

IOInitialize() {

BYTE IODevNdx;

/*--- global variables                                                       */

   Radix = 16;                         /* start out in HEX mode              */

   CmdShowMsg("Initializing device information\n");

   memset(IOFlags,0,sizeof(IOFlags));              /* assume no flags        */
   memset(IOInBits,0xff,sizeof(IOInBits));         /* assume all inputs      */

   CharDelay = CHAR_DELAY;                         /* set default delay      */

/*--- now check out each device to see if it is present                      */

   for (IODevNdx=0; IODevNdx<NUMDEVS; ++IODevNdx) {

      sprintf(CmdRespBuff," ...%s: ",IOName[IODevNdx]);
      CmdShowMsg(CmdRespBuff);

      IOFlags[IODevNdx][0] |= IO_INSTALLED;        /* Port 1 bits            */
      IOFlags[IODevNdx][0] |= IO_CPUPORT;          /* use special handling   */

      CmdShowMsg(" ready\n");
   }


}



/*---------------------------------------------------------------------------*/
/* Parse I/O device specifier into array index, channel offset, and bit ID   */
/* Argument is pointer to pointer to characters                              */
/* Returns index in high byte, channel in low byte                           */
/* Returns NODEVSPEC if device is no good                                    */
/* Always returns char ptr incremented past last char used                   */
/* This is hacked to return bit IDs instead of channels...                   */

int IOParseDev(ppSpec)
char **ppSpec;
{
WORD RetVal;
char *pBuffer;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
BYTE Temp;

   DevChan = DevBitID = 0;
   RetVal = NODEVSPEC | NOCHANSPEC | NOBITSPEC;    /* assume the worst       */

/*--- extract device string up to first non-alphanumeric character           */
/*    then look it up in the table for a match                               */

   memset(IOParseBuff,0,sizeof(IOParseBuff));
   pBuffer = IOParseBuff;
   while (isascii(**ppSpec) && isalnum(**ppSpec)) {         /* force caps    */
      *(pBuffer++) = toupper(**ppSpec);
      ++(*ppSpec);
   }

   for (DevNdx=0; DevNdx<NUMDEVS; ++DevNdx) {
      if (!strcmp(IOParseBuff,IOName[DevNdx])) {
         break;
      }
   }

   if (NUMDEVS == DevNdx) {
      sprintf(CmdRespBuff,"*** Bad device name [%s]\n",IOParseBuff);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }
   else {
      RetVal &= ~NODEVSPEC;
      if (REPORT_DEV & ReportMode) {
         sprintf(CmdRespBuff,"[Device %s",IOParseBuff);
         Nputstr(CmdRespBuff);
      }
   }


/*--- extract channel spec if we ended at a dot                              */

#ifdef x0

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (Temp < IONumChan[DevNdx]) {
         DevChan = Temp;
         RetVal &= ~NOCHANSPEC;
         if (REPORT_DEV & ReportMode) {
            sprintf(CmdRespBuff,", channel = %u",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         RetVal = NODEVSPEC;
         sprintf(CmdRespBuff,"\n*** Bad channel for %s: %u\n",
                  IOName[DevNdx],Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEV & ReportMode) {
         Nputstr(", all channels");
      }
   }

#else
   DevChan = 0;                        /* hack to force channel zero         */
   RetVal &= ~NOCHANSPEC;
#endif

/*--- extract bit spec if we are once again at a dot                         */

   if ('.' == **ppSpec) {
      ++(*ppSpec);
      Temp = GetDecimal(ppSpec);
      if (((Temp < 16) && (IO_WORD & IOFlags[DevNdx][0])) ||
          ((Temp <  8) && !(IO_WORD & IOFlags[DevNdx][0]))) {
         RetVal &= ~NOBITSPEC;
         DevBitID = Temp;
         if (REPORT_DEV & ReportMode) {
            sprintf(CmdRespBuff,", bit ID = %u]\n",Temp);
            Nputstr(CmdRespBuff);
         }
      }
      else {
         RetVal = NODEVSPEC;
         sprintf(CmdRespBuff,"\n*** Bad bit ID: %u\n",Temp);
         CmdShowMsg(CmdRespBuff);
         goto Done;
      }
   }
   else {
      if (REPORT_DEV & ReportMode) {
         Nputstr("]\n");
      }
   }

/*--- combine the parts and return a single word                             */
/*    preserves all the flags in the high nibble                             */
/*     (values will be zero if flags are set, but so it goes                 */

Done:

   RetVal |= (DevNdx << 8) | (DevChan << 4) | DevBitID;

   return RetVal;

}



/*---------------------------------------------------------------------------*/
/* Extract I/O device spec and value from string                             */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Extracts device index/channel/bit and returns the code                    */
/* Function return value is 0 for OK, 1 for error                            */

int IOSetValue(ppCmd,pDevCode)
char **ppCmd;
WORD *pDevCode;
{
int RetCode;
WORD DevCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
WORD Value;
WORD PortVal;

   RetCode = 1;                                    /* assume failure         */

   *pDevCode = DevCode = IOParseDev(ppCmd);        /* get device & channel   */
   if (NODEVSPEC & DevCode) {                      /* skip invalid devs      */
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   if (('=' != **ppCmd) || !(*((*ppCmd)+1))) {     /* check for =string      */
      CmdShowMsg("*** Set command missing data\n");
      goto Done;
   }

   ++(*ppCmd);                                     /* step over = sign       */

   if (NOCHANSPEC & DevCode) {
      CmdShowMsg("*** Missing channel\n");
   }
   else {                                          /* ordinary I/O string    */
      switch (Radix) {
      case 16 :
      default :
         Value = GetHexits(ppCmd,4);               /* get hex word if there  */
         break;
      case 10 :
         Value = GetDecimal(ppCmd);                /* get decimal            */
         break;
      }

      if (NOBITSPEC & DevCode) {                   /* missing bit spec?      */
         IOWritePort(DevNdx,DevChan,Value);        /* write whole byte       */
      }
      else {                                       /* have bit map spec      */
         if (Value > 1) {                          /* is data OK?            */
            CmdShowMsg("*** Bit value must be 0 or 1\n");
            goto Done;
         }
         if (IO_CPUPORT & IOFlags[DevNdx][0]) {    /* update input bit mask  */
            IOInBits[DevNdx][DevChan] &= ~(0x01 << DevBitID);
            IOInBits[DevNdx][DevChan] |= Value << DevBitID;
         }
         PortVal = IOReadPort(DevNdx,DevChan);     /* fetch existing data    */
         PortVal &= ~(0x01 << DevBitID);           /* remove old bit         */
         PortVal |= Value << DevBitID;             /* insert new bit         */
         PortVal |= IOInBits[DevNdx][DevChan];     /* mask off input bits    */
         IOWritePort(DevNdx,DevChan,PortVal);      /* write whole byte       */
      }
      RetCode = 0;
   }


Done:

   return RetCode;

}



/*---------------------------------------------------------------------------*/
/* Extract I/O device from incoming string                                   */
/* Returns value from appropriate port or bit                                */
/* Function return value is 0 for OK, 1 for error                            */
/* Returns string pointer ticked to just beyond the value... if any          */
/* Returns decoded value of device and channel                               */
/* If the device is not installed, displays error message                    */
/* This handles the clocks as a special case, returning zero with no         */
/*  channel specified...                                                     */

int IOGetValue(ppCmd,pDevCode,pValue)
char **ppCmd;
WORD *pDevCode;
WORD *pValue;
{

int RetCode;
WORD DevCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
WORD Value;

   *pDevCode = DevCode = IOParseDev(ppCmd);        /* get device & channel   */
   RetCode = 1;                                    /* assume error           */
   Value = 0;                                      /* just in case...        */

   if (NODEVSPEC & DevCode) {                      /* bad device             */
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   if (!(IO_INSTALLED & IOFlags[DevNdx][0])) {              /* installed?    */
      sprintf(CmdRespBuff,"*** Device %s not installed\n",IOName[DevNdx]);
      CmdShowMsg(CmdRespBuff);
      goto Done;
   }

   if ('=' == **ppCmd) {                           /* check for = sign!      */
      CmdShowMsg("*** Cannot set value\n");
      goto Done;
   }
   else {

      if (NOCHANSPEC & DevCode) {                  /* missing channel?       */
         DevChan = 0;                              /* default to zero here   */
      }

      if (NOBITSPEC & DevCode) {
        Value = IOReadPort(DevNdx,DevChan);
      }
      else {                                       /* use bit spec on data   */
         Value = IOReadPort(DevNdx,DevChan);       /* fetch existing data    */
         Value >>= DevBitID;                       /* move bit to LSB        */
         Value &= 0x0001;                          /* isolate it             */
      }
   }


   RetCode = 0;

Done:

   *pValue = Value;

#ifdef TRC_IO
   sprintf(CmdRespBuff,"Getval DevCode @%04x=%04x, Value @%04x=%02x\n",
            pDevCode,DevCode,pValue,Value);
   Nputstr(CmdRespBuff);
#endif

   return RetCode;

}



/*---------------------------------------------------------------------------*/
/* Formats value for given device and channel                                */
/* Produces appropriate text to identify the results                         */
/* Message for non-installed devices                                         */

int IOFormatData(DevCode,Value)
WORD DevCode;
WORD Value;
{
int RetCode;
BYTE DevNdx;
BYTE DevChan;
BYTE DevBitID;
char *pFormat;

   RetCode = 1;

#ifdef TRC_IO
   sprintf(CmdRespBuff,"<IOFormatData DevCode %04x, Value %02x>",
            DevCode,Value);
   Nputstr(CmdRespBuff);
#endif

   if (NODEVSPEC & DevCode) {
      strcpy(CmdRespBuff,"*** Bad device\n");
      goto Done;
   }

   DevNdx =   IO_DEVICE(DevCode);
   DevChan =  IO_CHANNEL(DevCode);
   DevBitID = IO_BITID(DevCode);

   strcpy(CmdRespBuff,IOName[DevNdx]);             /* set up device name     */

#ifdef x0

   if (!(NOCHANSPEC & DevCode)) {
      sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevChan);
      if (!(NOBITSPEC & DevCode)) {
         sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevBitID);
      }
   }
#else
   if (!(NOBITSPEC & DevCode)) {
      sprintf(CmdRespBuff+strlen(CmdRespBuff),".%u",DevBitID);
   }
#endif


   strcat(CmdRespBuff,"=");

   if (!(NOBITSPEC & DevCode)) {                   /* have bit specifier     */
      pFormat = "%u";                              /* either 0 or 1          */
   }
   else {
      switch (Radix) {
      case 16 :
      default :
         pFormat = (IO_WORD & IOFlags[DevNdx][0]) ? "%04x" : "%02x";
         break;
      case 10 :
         pFormat = "%u";
         break;
      }
   }

   sprintf(CmdRespBuff+strlen(CmdRespBuff),pFormat,Value);

   RetCode = 0;

Done:

   if (RetCode) {
      CmdShowMsg(CmdRespBuff);                     /* suppressable errors    */
   }
   else {
      Nputstr(CmdRespBuff);                        /* normal results         */
   }

   return RetCode;

}

/*---------------------------------------------------------------------------*/
/* Send text string to device                                                */

int IOSendString(pString,DevCode)
char *pString;
WORD DevCode;
{
int RetCode;
char RepString[10];
char *pRep;
char *npString;                                    /* need XRAM for GetHexit */

   while (*pString) {
      if ('\\' == *pString) {                      /* escape sequence?       */
         switch (tolower(*(pString+1))) {
         case 'c' :                                /* control chars          */
            RepString[0] = *(pString+2) & 0x1F;
            RepString[1] = 0;
            pString += 1;                          /* account for char       */
            break;
         case 'e' :                                /* escape char itself     */
            strcpy(RepString,"\x1B");
            break;
         case 'f' :                                /* formfeed               */
            strcpy(RepString,"\x0C");
            break;
         case 'n' :                                /* newline (LF and CR)    */
            strcpy(RepString,"\x0A");
            break;
         case 'r' :                                /* bare CR                */
            strcpy(RepString,"\x0D");
            break;
         case 'x' :                                /* direct hex output      */
            pRep = pString;                        /* save ptr to switch     */
            pString += 2;                          /* point to hexits        */
            npString = pString;
#ifdef TRC_ESC
            sprintf(CmdRespBuff,
                    "\n&pString=%04x pString=%04x *pString=%02x\n",
                       &pString,pString,*pString);
            sprintf(CmdRespBuff+strlen(CmdRespBuff),
                    "\n&npString=%04x npString=%04x *npString=%02x\n",
                       &npString,npString,*npString);
            Nputstr(CmdRespBuff);
#endif
            RepString[0] = (BYTE)GetHexits(&npString,2);
            RepString[1] = 0;
#ifdef TRC_ESC
            sprintf(CmdRespBuff,"Converted=%02x [%s] string in=[%s] out=[%s]\n",
                     RepString[0],RepString,pRep+2,npString);
            Nputstr(CmdRespBuff);
#endif
            pString = pRep + 2;                    /* restore ptr over chars */
            break;
         case '\\' :
            RepString[0] = '\\';                   /* send single slash      */
            RepString[1] = 0;
            break;
         default :
            RepString[0] = *(pString+1);           /* send next char anyway  */
            RepString[1] = 0;
         }
         pString += 2;
         pRep = RepString;
         while (*pRep) {
            IOWritePort(IO_DEVICE(DevCode),IO_CHANNEL(DevCode),*pRep);
            ++pRep;
            if (CharDelay) {
               TmrDelay100US(CharDelay);
            }
         }
      }
      else {
         IOWritePort(IO_DEVICE(DevCode),IO_CHANNEL(DevCode),*pString);
         ++pString;
         if (CharDelay) {
            TmrDelay100US(CharDelay);
         }
      }

   }

   return RetCode;

}

