/* COMMAND.C -- command decoder                                              */
/* Common routines for direct serial connection                              */


/*---------------------------------------------------------------------------*/
/* Initialize command decoder                                                */

CmdInitialize() {

   CmdBits = 0;

   CmdLineLen = 0;
   memset(CmdBuffer,0,sizeof(CmdBuffer));

   CmdRespLen = 0;
   memset(CmdRespBuff,0,sizeof(CmdBuffer));

   ReportMode = REPORT_NONE;                       /* disable log outputs    */
}


/*---------------------------------------------------------------------------*/
/* Assemble complete command in buffer                                       */
/* Returns NULL until a \n shows up, then returns address of buffer          */
/* The command is terminated in 0 rather than a newline                      */

char *CmdGetLine() {

char NewChar;
char *RetVal;

   RetVal = NULL;                      /* assume not end of line             */

   if (chkch()) {
      NewChar = getch();               /* fetch the character                */

#ifdef CMD_TRACE
      printf("[%u %s %c]\n",CmdLineState,CmdBuffer,NewChar);
#endif

      Nputch(NewChar);
      if ('\n' == NewChar) {
            CmdBuffer[CmdLineLen] = 0;             /* ensure terminator      */
            CmdLineLen = 0;                        /* for next time around   */
            RetVal = CmdBuffer;                    /* return buffer address  */
      }
      else {
         CmdBuffer[CmdLineLen] = NewChar;
         if (MAXCMDLEN <= ++CmdLineLen) {
            CmdLineLen = MAXCMDLEN - 1;
         }
         switch (NewChar) {
         case BS :
            --CmdLineLen;                          /* discard BS             */
            CmdBuffer[CmdLineLen] = 0;
            if (CmdLineLen) {
               --CmdLineLen;                       /* back up one slot       */
               CmdBuffer[CmdLineLen] = 0;          /* discard that char      */
            }
            break;
         default :
            if (1 == CmdLineLen) {
               memset(CmdBuffer+1,0,sizeof(CmdBuffer)-1);
            }
         }
      }
   }

   return RetVal;


}


/*---------------------------------------------------------------------------*/
/* Various command decoder routines                                          */
/* These are invoked from all the separate decoders to cut down on dupes     */

/*--- set the network address                                                */

CmdDecSetAddr(pCmd)
char *pCmd;
{

   pCmd = pCmd;

   Nputstr("*** Network only\n");

}


/*--- dump common internal status                                            */

CmdDecDumpState(pCmd)
char *pCmd;
{

   pCmd = pCmd;

   Nputstr("Interactive mode");

 asm {
      MOV   R0,#LOW(MAXSTACK+1)                    ; check stack from top
?slp
      DEC   R0
      MOV   A,[R0]
      JZ    ?slp
      MOV   TestBits,R0
 }

   sprintf(CmdRespBuff,".  Stack used %02x/%02x.\n",
          TestBits,MAXSTACK);
   Nputstr(CmdRespBuff);

   SerDumpState();

}

/*--- dump error flags                                                       */

CmdDecDumpErrs(pCmd)
char *pCmd;
{

   pCmd = pCmd;

   sprintf(CmdRespBuff,"Error flags %02x\n",ErrFlags);
   Nputstr(CmdRespBuff);
   ErrFlags = 0;

}


/*--- set or report logging mode                                             */

CmdDecLogging(pCmd)
char *pCmd;
{

char *pChar;

   pChar = pCmd + 1;

   if (isdigit(*pChar)) {
      ReportMode = atoi(pChar);
   }
   else {
      sprintf(CmdRespBuff,"Logging mode is %u\n",ReportMode);
      Nputstr(CmdRespBuff);
   }

}

/*--- set or report network mode                                             */

CmdDecNetwork(pCmd)
char *pCmd;
{

   pCmd = pCmd;
   Nputstr("*** Network only\n");

}


/*--- report an invalid command                                              */

CmdDecNoCmd(pCmd)
char *pCmd;
{

   sprintf(CmdRespBuff,"*** Unknown command: [%s]\n",pCmd);
   CmdShowMsg(CmdRespBuff);

}

