/* ANSI.C                                                                    */

#define ANSI_TRACE

/*---------------------------------------------------------------------------*/
/* Initialize the ANSI sequence decoder                                      */

ANSIInitialize() {

   ANSIState = ANSI_WAIT;
   ANSICharCtr = ANSIParamCtr = 0;

   ANSISavedRow = ANSISavedCol = 1;

   return;

}



/*---------------------------------------------------------------------------*/
/* Accept a character and drive the state machine                            */

ANSIDriver(NewChar)
WORD NewChar;
{

   if (REPORT_ANSI & ReportMode) {
      sprintf(CmdRespBuff,"--- ANSI char is %02x [%c], state %u\n",
         NewChar,isprint(NewChar) ? NewChar : '-',
         ANSIState);
      Nputstr(CmdRespBuff);
   }

   if (ANSI_OFF == ANSIState) {        /* if driver off,                     */
      LCDSendChar(NewChar);            /* send directly to LCD panel         */
      return;                          /* and be done with it...             */
   }

   ANSIBuffer[ANSICharCtr] = NewChar & 0x7F;       /* strip & save           */
   ANSICharCtr = min(++ANSICharCtr,ANSI_MAXLEN-1);

/*--- decide what to do with char based on current state                     */

   switch (ANSIState) {
   case ANSI_WAIT :                    /* waiting for ESC char               */
      ANSICharCtr = ANSIParamCtr = 0;              /* force buffer restart   */
      if (ESC == NewChar) {
         if (REPORT_ANSI & ReportMode) {
            Nputstr("   Start ANSI sequence\n");
         }
         memset(ANSIBuffer,0,sizeof(ANSIBuffer));
         ANSIState = ANSI_MODE;
      }
      else {
         LCDSendChar(NewChar);         /* just dump it                       */
      }
      break;
   case ANSI_MODE :                    /* expect bracket as next char        */
      if ('[' == NewChar) {
         ANSIState = ANSI_PARAM;
      }
      else {
         if (REPORT_ANSI & ReportMode) {
            Nputstr("    Missing [ char, sequence cancelled\n");
         }
         ANSINull(NewChar);                        /* ditch sequence         */
      }
      break;
   case ANSI_PARAM :
      if ('=' == NewChar) {
         ANSIState = ANSI_PARAM;                   /* start of params        */
      }
      else {
         if (isdigit(NewChar)) {
            ANSIState = ANSI_NUM;                  /* still a number         */
         }
         else {
            if (isalpha(NewChar)) {
               ANSIGetNumber();                    /* get number if any      */
               ANSIDoCmd(NewChar);                 /* end of sequence        */
            }
            else {
               if (REPORT_ANSI & ReportMode) {
                  sprintf(CmdRespBuff,
                          "    Bad char %02x [%c], cancelled\n",
                          NewChar, isprint(NewChar) ? NewChar : '-');
                  Nputstr(CmdRespBuff);
               }
               ANSINull(NewChar);
            }
         }
      }
      break;
   case ANSI_NUM :
      if (';' == NewChar) {                        /* end of param digit     */
         ANSIGetNumber();                          /* save prev param        */
      }
      else {
         if (isdigit(NewChar)) {
            ANSIState = ANSI_NUM;                  /* still a number         */
         }
         else {
            if (isalpha(NewChar)) {
               ANSIGetNumber();                    /* get number if any      */
               ANSIDoCmd(NewChar);                 /* end of sequence        */
            }
            else {
               if (REPORT_ANSI & ReportMode) {
                  sprintf(CmdRespBuff,
                          "    Bad char %02x [%c] after param, cancelled\n",
                          NewChar, isprint(NewChar) ? NewChar : '-');
                  Nputstr(CmdRespBuff);
               }
               ANSINull(NewChar);
            }
         }
      }
      break;
   default :
      ANSIInitialize();                            /* error, restart decoder */
   }


}

/*---------------------------------------------------------------------------*/
/* Extract a decimal number from the stored escape sequence                  */
/* Stored sequence included the char that triggered this, so we must back    */
/*  up over that character to begin the search                               */
/* Updates the array of parameters for easy access                           */

int ANSIGetNumber() {
int TempNum;
char *pChar;

/* find start of numeric string                                              */

   if (REPORT_ANSI & ReportMode) {
      sprintf(CmdRespBuff,"   Extracting param from [%s] @ %u: ",
         ANSIBuffer,ANSICharCtr-2);
      Nputstr(CmdRespBuff);
   }

   for (pChar = ANSIBuffer + ANSICharCtr - 2;
        pChar >= ANSIBuffer;
        --pChar) {
      if (!isdigit(*pChar)) {
         ++pChar;
         if (REPORT_ANSI & ReportMode) {
            sprintf(CmdRespBuff,"(Num string is [%s])  ",
               pChar);
            Nputstr(CmdRespBuff);
         }
         break;
      }
   }

/* do the conversion                                                         */
/* if the number is missing, we default to 1 rather than zero                */

   if (isdigit(*pChar)) {
      TempNum = atoi(pChar);
   }
   else {
      TempNum = 1;
   }

/* update the array                                                          */

   ANSIParams[ANSIParamCtr] = TempNum;
   ANSIParamCtr = min(++ANSIParamCtr,ANSI_MAXNUMS-1);


   if (REPORT_ANSI & ReportMode) {
      sprintf(CmdRespBuff," %u\n",TempNum);
      Nputstr(CmdRespBuff);
   }

   return TempNum;

}



/*---------------------------------------------------------------------------*/
/* Driver state routines                                                     */

/*--- mismatch handler... dump stored chars & reset to beginning             */

ANSINull(NewChar)
WORD NewChar;
{

   NewChar = NewChar;                  /* do not need this, already stored   */

   if (ANSICharCtr) {
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Dumping sequence [%s]\n",ANSIBuffer);
      }
      LCDSendChar(ESC);                /* not stored in buffer               */
      LCDSendString(ANSIBuffer);       /* all chars including current one    */
   }

   ANSICharCtr = ANSIParamCtr = 0;
   ANSIState = ANSI_WAIT;

}



/*--- hit end of sequence, so do the appropriate command action routine      */

ANSIDoCmd(NewChar)
WORD NewChar;
{
BYTE Counter;

   if (REPORT_ANSI & ReportMode) {
      Nputstr("   Start decode:  ");
   }

   ANSIState = ANSI_WAIT;                          /* by default...          */

   switch (NewChar) {
   case 'A' :                          /* cursor up                          */
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Cursor up %u ",
            ANSIParams[0]);
         Nputstr(CmdRespBuff);
      }
      LCDSetCursor(CurrentRow-ANSIParams[0],CurrentCol);
      break;
   case 'B' :                          /* cursor down                        */
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Cursor down %u ",
            ANSIParams[0]);
         Nputstr(CmdRespBuff);
      }
      LCDSetCursor(CurrentRow+ANSIParams[0],CurrentCol);
      break;
   case 'C' :                          /* cursor forward                     */
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Cursor forward %u ",
            ANSIParams[0]);
         Nputstr(CmdRespBuff);
      }
      LCDSetCursor(CurrentRow,CurrentCol+ANSIParams[0]);
      break;
   case 'D' :                          /* cursor back                        */
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Cursor back %u ",
            ANSIParams[0]);
         Nputstr(CmdRespBuff);
      }
      LCDSetCursor(CurrentRow,CurrentCol-ANSIParams[0]);
      break;
   case 'H' :                          /* set cursor position                */
   case 'f' :                          /* set cursor position (synonym)      */
   case 'j' :                          /* set cursor position (synonym)      */
      for (;ANSIParamCtr < 2; ++ANSIParamCtr) {
         ANSIParams[ANSIParamCtr] = 1;             /* force defaults         */
      }
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Cursor to ANSI row %u, col %u ",
            ANSIParams[0],ANSIParams[1]);
         Nputstr(CmdRespBuff);
      }
      LCDSetCursor(ANSIParams[0]-1,ANSIParams[1]-1);        /* fix origin!   */
      break;
   case 'J' :                          /* erase display & home cursor        */
      if (2 == ANSIParams[0]) {
         if (REPORT_ANSI & ReportMode) {
            Nputstr("   Clear and home cursor");
         }
         LCDClear();
      }
      break;
   case 'K' :                          /* erase to end of line               */
      if (REPORT_ANSI & ReportMode) {
         Nputstr("   Clear to end of line");
      }
      Counter = VisibleCols - CurrentCol;
      while (Counter) {
         LCDSendChar(DEL);
         --Counter;
      }
      break;
   case 'R' :                          /* cursor location report response    */
      break;                           /*  we ignore this if we get it       */
   case 'h' :                          /* set screen mode, wrap at line end  */
      if (7 == ANSIParams[0]) {
         if (REPORT_ANSI & ReportMode) {
            Nputstr("   Set line wrap mode");
         }
         LCDControls |= LCD_HWRAP;
      }
      break;
   case 'l' :                          /* set screen mode, no wrap at end    */
      if (7 == ANSIParams[0]) {
         if (REPORT_ANSI & ReportMode) {
            Nputstr("   Clear line wrap mode");
         }
         LCDControls &= ~LCD_HWRAP;
      }
      break;
   case 'm' :                          /* set graphics rendition             */
      break;                           /*  this is ignored                   */
   case 'n' :                          /* query cursor location              */
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Query cursor location (row %u, col %u)",
                  CurrentRow+1,CurrentCol+1);
         Nputstr(CmdRespBuff);
      }
      sprintf(CmdRespBuff,"\x1b[%u;%uR\n",CurrentRow+1,CurrentCol+1);
      Nputstr(CmdRespBuff);
      break;
   case 'p' :                          /* reassign key values                */
      break;
   case 's' :                          /* save cursor location               */
      ANSISavedRow = CurrentRow+1;
      ANSISavedCol = CurrentCol+1;
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Save cursor (ANSI row %u, col %u)",
                  ANSISavedRow,ANSISavedCol);
         Nputstr(CmdRespBuff);
      }
      break;
   case 'u' :                          /* restore cursor location            */
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Restore cursor (ANSI row %u, col %u)",
                  ANSISavedRow,ANSISavedCol);
         Nputstr(CmdRespBuff);
      }
      LCDSetCursor(ANSISavedRow-1,ANSISavedCol-1);
      break;
   default :
      if (REPORT_ANSI & ReportMode) {
         sprintf(CmdRespBuff,"   Invalid command char [%c] %02x hex",
                  iscntrl(NewChar)?NewChar:'-',NewChar);
         Nputstr(CmdRespBuff);
      }
      ANSINull(NewChar);               /* dump all others...                 */
   }

   if (REPORT_ANSI & ReportMode) {
      Nputstr("\n");
   }

}


/*---------------------------------------------------------------------------*/
/* Send a string to the ANSI decoder                                         */
/* We decode a few C-style formatting characters...                          */

ANSISendString(pString)
char *pString;
{
char RepString[10];
char *pRep;
char *xpChar;                                      /* must be in XRAM        */
BYTE Temp;
BYTE OldRow,OldCol,NewRow,NewCol;

   while (*pString) {
      if ('\\' == *pString) {                      /* escape sequence?       */
         switch (tolower(*(pString+1))) {
         case 'b' :                                /* bell sound             */
            RepString[0] = 0;                               /* not to LCD    */
            BeepCtr = BEEPTIME;                             /* start output  */
            break;
         case 'c' :                                /* control chars          */
            RepString[0] = *(pString+2) & 0x1F;
            RepString[1] = 0;
            pString += 1;                          /* account for char       */
            break;
         case 'e' :                                /* escape char itself     */
            strcpy(RepString,"\x1B");
            break;
         case 'f' :                                /* formfeed               */
            strcpy(RepString,"\x0C");
            break;
         case 'n' :                                /* newline (LF and CR)    */
            strcpy(RepString,"\x0A");
            break;
         case 'r' :                                /* bare CR                */
            strcpy(RepString,"\x0D");
            break;
         case 't' :                                /* tab                    */
            Temp = TABINTERVAL - ((CurrentCol+1) % TABINTERVAL);
            memset(RepString,' ',Temp);
            RepString[Temp] = 0;
            break;
         case 'x' :                                /* direct hex output      */
            OldRow = CurrentRow;
            OldCol = CurrentCol;
            ANSIDriver(" ");                       /* fake a single blank    */
            NewRow = CurrentRow;
            NewCol = CurrentCol;
            xpChar = pString + 2;                  /* aim at hexits          */
            Temp = GetHexits(&xpChar,2);           /* xpChar may get ticked  */
            LCDSetCursor(OldRow,OldCol);           /* back to starting point */
            LCDSendByte(Temp,LCDWAIT|LCDDATA);     /* show the byte          */
            LCDSetCursor(NewRow,NewCol);           /* to new location        */
            RepString[0] = 0;
            pString += 2;                          /* restore ptr over chars */
            break;
         case '\\' :
            RepString[0] = '\\';                   /* send single slash      */
            RepString[1] = 0;
            break;
         default :
            RepString[0] = *(pString+1);           /* send next char anyway  */
            RepString[1] = 0;
         }
         pString += 2;
         pRep = RepString;
         while (*pRep) {
            ANSIDriver(*pRep);
            ++pRep;
         }
      }
      else {
         ANSIDriver(*pString);                     /* just copy the char     */
         ++pString;
      }


   }

}

