/* COMMAND.C -- command decoder                                              */
/* Also includes network address recognition & response code                 */

/* #define CMD_TRACE */

static char RawBuffer[MAXCMDLEN+1];    /* saved un-capped command string     */


/*---------------------------------------------------------------------------*/
/* Create network address from port bits                                     */
/* This is a dummy, as there are no bits left over...                        */

#ifdef NETWORK

int CmdMakeAddress() {

   return 0;

}

#endif


/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */

#ifdef NETWORK

int CmdCheckMode() {

asm {
   SETB  MANUALMODE                    ; ensure input on bit
   MOV   C,MANUALMODE                  ; pick up manual mode input
   CPL   C                             ; flip so jumper installed = 1
   CLR   A                             ; start with zero
   MOV   A.0,C                         ; combine with control bit
   MOV   TestBits,A                    ; set up for C code
}

   if (TestBits) {
      CmdNetMode(0);                   /* jumper installed sets manual mode  */
   }
   else {
      CmdNetMode(NETMODE);             /* all other addresses are automatic  */
   }

   return;

}

#endif



/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
int ArgVal;

   strcpy(RawBuffer,CmdBuffer);        /* preserve text for S command        */

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'D' :                          /* dump internal status               */
      CmdDecDumpState();
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      break;
   case 'L' :                          /* set debug logging options          */
      CmdDecLogging(pCmd);
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'O' :                          /* set output bit                     */
      ArgVal = (int)*(pChar+2) - '0';              /* state 0/1              */
      if (('=' == *(pChar+1)) &&
          ((0 <= ArgVal) && (1 >= ArgVal))) {
         switch (*pChar) {                         /* Bit selector           */
         case 'A' :
            if (ArgVal) {
               SETBIT(OUT_A);
            }
            else {
               CLRBIT(OUT_A);
            }
            break;
         case 'B' :
            if (ArgVal) {
               SETBIT(OUT_B);
            }
            else {
               CLRBIT(OUT_B);
            }
            break;
         case 'C' :
            if (ArgVal) {
               SETBIT(OUT_C);
            }
            else {
               CLRBIT(OUT_C);
            }
            break;
         default :
            CmdShowMsg("*** Output:  bad bit ID\n");
         }
      }
      else {
         CmdShowMsg("*** Output:  bad format\n");
      }
      break;
   case 'Q' :                          /* query keypad inputs                */
      SETBIT(FlagPortBusy);                        /* prevent changes        */
      sprintf(CmdRespBuff,"%02x\n",KeysHit);
      KeysHit = 0;                                 /* reset memory           */
      CLRBIT(FlagPortBusy);
      Nputstr(CmdRespBuff);
      break;
   case 'R' :
      if (strcmp(pChar,"ESET")) {
         CmdShowMsg("*** Unknown command\n");
      }
      else {
         SerWaitOutDone();
 asm {
   LJMP   0                            ; solid reset...
 }
      }
      break;
   case 'S' :                          /* send message to LCD panel          */
      pChar = strchr(RawBuffer,'=');               /* locate in saved string */
      if (pChar && ('=' == *pChar) && (1 < strlen(pChar))) {
         ANSISendString(pChar+1);
      }
      else {
         CmdShowMsg("*** S command string is missing\n");
      }
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      Nputstr("Available commands (values in decimal unless noted\n");
      Nputstr(" A=string  +  set network address to string\n");
      Nputstr(" D         *  dump internal status variables (some in hex)\n");
      Nputstr(" E         *  show and clear error flags\n");
      Nputstr(" L n          set debugging output mode\n");
      Nputstr(" N n      *+  set network mode\n");
      Nputstr(" O x=n        set output bit x to state n\n");
      Nputstr(" Q            query keys pressed\n");
      Nputstr(" RESET        perform power-on reset initialization\n");
      Nputstr(" S=str        show string on LCD w/ANSI processing\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;


}

