/* COMMAND.C -- command decoder                                              */

/*
#define CMD_TRACE
*/

/*---------------------------------------------------------------------------*/
/* Create network address from P1 bits                                       */
/* The next-to-last character of the string gets the P1 digit                */
/*  (the string should always end in a blank, but we do not check for this)  */

#ifdef NETWORK

int CmdMakeAddress() {

BYTE AddrBits;
BYTE AddrChar;
BYTE AddrLen;

   AddrBits = ((~P1) & 0x0E) >> 1;     /* get input bits, flip, & align      */
   AddrChar = AddrBits | '0';          /* turn into character                */

   AddrLen = strlen(CmdNetAddress);    /* insert it in the string            */
   if (2 <= AddrLen) {
      CmdNetAddress[AddrLen - 2] = AddrChar;
   }
   else {
      CmdNetAddress[0] = AddrChar;
   }

   return (int)AddrBits;

}

#endif


/*---------------------------------------------------------------------------*/
/* Check for manual mode                                                     */

#ifdef NETWORK

int CmdCheckMode() {

   SETBIT(MANUALMODE);                 /* allow input on bit                 */
   TESTBIT(TestBits,MANUALMODE);       /* decide if it is on or off          */

   if (TestBits) {
      CmdNetMode(NETMODE);             /* bit ON = no jumper = network       */
   }
   else {
      CmdNetMode(0);                   /* bit OFF = jumper = manual mode     */
   }

   return;

}

#endif


/*---------------------------------------------------------------------------*/
/* Decode commands                                                           */

CmdDecode(pCmd)
char *pCmd;
{
char *pChar;
char *pArg;
int ArgVal;

   pChar = pArg = pCmd;                /* uppercase & squash blanks          */
   while (*pChar) {
      if (' ' == *pChar) {                         /* skip blanks here       */
         ++pChar;
      }
      else {
         *pArg = toupper(*pChar);
         ++pChar;                                  /* copy the rest          */
         ++pArg;
      }
   }
   *pArg = 0;                          /* force terminator after squash      */
   pChar = pCmd+1;                     /* set up for arg if any              */

   switch (*pCmd) {
   case 'A' :                          /* dump or set network address        */
      CmdDecSetAddr(pCmd);
      break;
   case 'B' :                          /* badge response delay               */
      if (isdigit(*pChar)) {
         BadgeRsp = atoi(pChar);
      }
      else {
         sprintf(CmdRespBuff,"Badge response delay is %u ticks\n",
                 BadgeRsp);
         Nputstr(CmdRespBuff);
      }
      break;
   case 'C' :                          /* calibrate oscillators              */
      switch (*pChar) {
      case 'R' :
         IRCalRemote(atoi(pChar+1));
         break;
      case 'T' :
         IRCalTrans(atoi(pChar+1));
         break;
      default :
         CmdShowMsg("*** Bad Calibrate command\n");
      }
      break;
   case 'D' :                          /* dump internal status               */
      CmdDecDumpState();
      IRDumpState();
      PolDumpState();
      MCDumpState();
      break;
   case 'E' :                          /* show error flags                   */
      CmdDecDumpErrs(pCmd);
      break;
   case 'H' :                          /* HEX send/receive                   */
      switch (*pChar++) {
      case 'D' :                       /* dump HEX from RAM                  */
         ArgVal = isxdigit(*pChar) ? GetHexits(&pChar,4) : pMCEPROM;
         MCDumpData(pMCRAM,ArgVal);
         break;
      case 'E' :                       /* dump HEX from EPROM                */
         ArgVal = isxdigit(*pChar) ? GetHexits(&pChar,4) : pMCEPROM;
         MCDumpData(pMCEPROM,ArgVal);
         break;
      case 'R' :                       /* receive HEX to RAM                 */
         MCLoadRAM();
         break;
      default :
         CmdShowMsg("*** Unknown HEX command\n");
      }
      break;
   case 'I' :                          /* badge polling interval             */
      if (isdigit(*pChar)) {
         PollInterval = atoi(pChar);
         if (PollInterval < 20) {                  /* force minimum delay    */
            PollInterval = 20;
            sprintf(CmdRespBuff,"*** Badge polling interval forced to %u\n",
                       PollInterval);
            CmdShowMsg(CmdRespBuff);
         }
      }
      else {
         sprintf(CmdRespBuff,"Badge polling interval is %u ticks\n",
                 PollInterval);
         Nputstr(CmdRespBuff);
      }
      break;
   case 'L' :                          /* set receiver logging modes         */
      CmdDecLogging(pCmd);
      break;
   case 'N' :                          /* select net mode                    */
      CmdDecNetwork(pCmd);
      break;
   case 'O' :                          /* set output bit                     */
      ArgVal = (int)*(pChar+2) - '0';              /* state 0/1              */
      if (('=' == *(pChar+1)) &&
          ((0 <= ArgVal) && (1 >= ArgVal))) {
         switch (*pChar) {                         /* Bit selector           */
         case 'A' :
            if (ArgVal) {
               SETBIT(OUT_A);
            }
            else {
               CLRBIT(OUT_A);
            }
            break;
         case 'B' :
            if (ArgVal) {
               SETBIT(OUT_B);
            }
            else {
               CLRBIT(OUT_B);
            }
            break;
         case 'C' :
            if (ArgVal) {
               SETBIT(OUT_C);
            }
            else {
               CLRBIT(OUT_C);
            }
            break;
         default :
            CmdShowMsg("*** Output:  bad bit ID\n");
         }
      }
      else {
         CmdShowMsg("*** Output:  bad format\n");
      }
      break;
   case 'P' :                          /* set number of badges to poll       */
      if (isdigit(*pChar)) {
         PolNumIDs = atoi(pChar);
         PolNextID = 0;                /* restart from badge zero            */
      }
      else {
         sprintf(CmdRespBuff,"Number of badges is %u\n",PolNumIDs);
         Nputstr(CmdRespBuff);
      }
      break;
   case 'Q' :                          /* query IR responses                 */
      if (*pChar) {
         pArg = strchr(pChar,'-');
         if (NULL == pArg) {
            if (isdigit(*pChar)) {
               ArgVal = atoi(pChar);
               IRDumpMsg(ArgVal,ArgVal);
            }
            else {
               CmdShowMsg("*** Query:  bad param\n");
            }
         }
         else {
            ArgVal = atoi(pChar);
            IRDumpMsg(ArgVal,atoi(pArg+1));
         }
      }
      else {
         IRDumpMsg(0,MAXMSGNUM);       /* no param = full dump               */
      }
      break;
   case 'R' :
      if (strcmp(pChar,"ESET")) {
         CmdShowMsg("*** Unknown command\n");
      }
      else {
         SerWaitOutDone();
 asm {
   LJMP   0                            ; solid reset...
 }
      }
      break;
   case 'S' :                          /* send IR message                    */
      if (isdigit(*pChar)) {
         IRSendMsg(atoi(pChar));
      }
      else {
         if ('M' == *pChar) {
            MCSendSignal(atoi(pChar+1));
         }
         else {
            CmdShowMsg("*** Missing parameter\n");
         }
      }
      break;
   case 'T' :                          /* train a new MCIR signal            */
      if (isdigit(*pChar)) {
         MCCapture(atoi(pChar));
      }
      else {
         switch (*pChar) {
         case 'D' :                    /* adjust duty cycle                  */
            if (isdigit(*++pChar)) {
               MCDuty = atoi(pChar);
            }
            sprintf(CmdRespBuff,"Duty cycle correction is %u cycles\n",
                     MCDuty);
            Nputstr(CmdRespBuff);
            break;
         case 'X' :
            MCInitData();
            break;
         default :
            CmdShowMsg("*** Missing MCIR record number\n");
         }
      }
      break;
   case 0 :                            /* ignore empy line                   */
      break;
   case '?' :
      Nputstr("Available commands (values in decimal unless noted)\n");
      Nputstr(" (times in 5.16 ms ticks)\n");
      Nputstr(" A=str    + set network address\n");
      Nputstr(" B n        set badge response time delay\n");
      Nputstr(" CR n       show MC145030 osc period, n = report detail\n");
      Nputstr(" CT         calibrate transmit oscillator (U1.11 to T1)\n");
      Nputstr(" D        * dump internal status variables (some in hex)\n");
      Nputstr(" E        * show and clear error flags\n");
      Nputstr(" I n        set badge polling interval\n");
      Nputstr(" HD addr    dump MCIR data from RAM to HEX w/base addr\n");
      Nputstr(" HE         as HD, but from EPROM\n");
      Nputstr(" HR         receive MCIR data from HEX to RAM\n");
      Nputstr(" L n        set reporting mode (bitmapped)\n");
      Nputstr(" N n     *+ set network mode\n");
      Nputstr(" P n        set number of badges to poll\n");
      Nputstr(" Q          dump all received MC145030 ID codes in hex\n");
      Nputstr(" Q n        dump single MC145030 ID in binary\n");
      Nputstr(" Q n-n      dump range of MC145030 IDs in hex\n");
      Nputstr(" RESET      perform power-on reset initialization\n");
      Nputstr(" S n        send MC145030 IR ID code n\n");
      Nputstr(" SM n       send MCIR signal n (1-999 RAM, 1000-1999 EPROM)\n");
      Nputstr(" T n        train new MCIR code, store in RAM as n (1-999)\n");
      Nputstr(" TD n       set duty cycle correction factor 0-255\n");
      Nputstr(" TX         clear MCIR data from RAM\n");
      Nputstr("+ marks network-only commands\n");
      Nputstr("* marks debugging commands\n");
      break;
   default :
      CmdDecNoCmd(pCmd);
   }

   return;


}

