@ECHO OFF
CLS
ECHO Micro-C 8051 compile: %1
REM ----------------------
REM set up a few defaults
IF "%mclib%" == "" SET mclib=%mcdir%
IF "%mcinclude%" == "" SET mclib=%mcdir%
IF "%mcromend%" == "" SET mcromend=7fff
SET model=%mcmodel%
IF "%model%" == "" SET model=tiny
SET discard=yes
SET quiet=-q
SET beeps=1
REM ----------------------
REM extract command line parameters
IF "%1" == "" goto help
SET target=%1
SHIFT
:parm1
IF "%1" == "keep" SET discard=no
IF "%1" == "keep" GOTO parm2
IF NOT "%1" == "" SET model=%1
:parm2
SHIFT
IF NOT "%1" == "" GOTO parm1
:parm9
REM ----------------------
REM sort out memory model settings
SET cmem=x
IF %model% == tiny     SET cmem=0
IF %model% == small    SET cmem=1
IF %model% == compact  SET cmem=2
IF %model% == hcs8000  SET cmem=2
IF %model% == hcsa000  SET cmem=2
IF %model% == medium   SET cmem=3
IF %model% == large    SET cmem=4
IF NOT %cmem% == x GOTO run
ECHO *** Invalid memory model [%model%]
REM ----------------------
:help
ECHO ---
ECHO Syntax is %0 filename [memorymodel] [keep]
ECHO  Don't include the extension on the filename
ECHO  memorymodel:  overrides MCMODEL environment variable
ECHO  keep:  preserves temporary files in %mctmp%
ECHO  (lower case for everything...)
ECHO ---
ECHO These environment variables are useful:
ECHO  MCDIR      subdirectory containing executable files       %mcdir%
ECHO  MCTMP      subdirectory for temporary files               %MCTMP%
ECHO  MCLIB      subdirectory for 8051 library files            %MCLIB%
ECHO  MCINCLUDE  subdirectory for 8051 header files             %MCINCLUDE%
ECHO  MCCUSTOM   subdirectory for customized files              %MCCUSTOM%
ECHO  MCMODEL    memory model:  tiny/small/medium/compact/large %MCMODEL%
ECHO  MCMODEXT   suffix for memory model filename               %MCMODEXT%
ECHO ---
goto done
REM ----------------------
REM Do the deed...
:run
ECHO --- clean up old files
IF exist %mctmp%.\%target%.CP  del %mctmp%.\%target%.CP
IF exist %mctmp%.\%target%.AS* del %mctmp%.\%target%.AS*
IF exist %mctmp%.\%target%.OPT del %mctmp%.\%target%.OPT
IF exist          %target%.LST del          %target%.LST
IF exist %target%.HEX del %target%.HEX
ECHO --- C preprocessor
REM don't use -c option... MCP complains about apostrophes in asm comments
MCP %target%.c %mctmp%.\%target%.CP l=%mcinclude% %quiet% -c
IF ERRORLEVEL 1 GOTO errors
ECHO --- C compiler [memory model m=%cmem%]
MCC51 %mctmp%.\%target%.CP %mctmp%.\%target%.AS1 %quiet% m=%cmem%
IF ERRORLEVEL 1 GOTO errors
IF "%discard%" == "yes" DEL %mctmp%.\%target%.CP
ECHO --- ASM preprocessor
MACRO %mccustom%\macros.inc %mctmp%.\%target%.AS1 > %mctmp%.\%target%.AS2
ECHO --- optimizer
MCO51 %mctmp%.\%target%.AS2 %mctmp%.\%target%.AS3 %quiet% -d > %mctmp%.\%target%.OPT
IF ERRORLEVEL 1 GOTO errors
IF "%discard%" == "yes" DEL %mctmp%.\%target%.AS1
IF "%discard%" == "yes" DEL %mctmp%.\%target%.AS2
IF "%discard%" == "yes" DEL %mctmp%.\%target%.OPT
ECHO --- source linker [using library %model%%mcmodext%.lib]
SLINK %mctmp%.\%target%.AS3 %mctmp%.\%target%.ASM %quiet% i=%model%%mcmodext%.lib t=%mctmp% l=%mclib%
IF ERRORLEVEL 1 GOTO errors
IF "%discard%" == "yes" DEL %mctmp%.\%target%.AS3
ECHO --- assembler
ASM51 %mctmp%.\%target%.ASM -fis l=%target%.lst c=%mctmp%.\%target%.hxx
IF ERRORLEVEL 1 GOTO errors
IF "%discard%" == "yes" DEL %mctmp%.\%target%.ASM
ECHO --- checksum
HEXFMT %mctmp%.\%target%.hxx -i s=$8000 c=$0000,$%mcromend%,$002E w=%target%.hex
IF ERRORLEVEL 1 GOTO errors
IF "%discard%" == "yes" DEL %mctmp%.\%target%.HXX
REM ----------------------
REM This is useful if you have an EPROM emulator
REM The command used here is for the Parallax 32K ROM emulator
IF NOT "%download%" == "yes" GOTO done
ECHO --- Parallax EPROM emulator HEX file transfer
PRE %target%.HEX
ECHO --- Done!
GOTO done
:errors
ECHO *** Got problems, mon...
SET beeps=1 2
:done
for %%f IN (%beeps%) do echo 
REM ----------------------
REM Clean up the environment
SET beeps=
SET cmem=
SET discard=
SET model=
SET quiet=
SET target=
