/* ACLINE.C -- power line interface                                          */

/*
#define PWR_TRACE
*/

/*---------------------------------------------------------------------------*/
/* Set up timers and suchlike                                                */

PwrInitialize() {

/*--- interrupt vectors                                                      */

 asm {
PWRHERE EQU *
   ORG   $0003                   /* External Interrupt 0  (AC line sync)     */
   LJMP  ExtInt0
   ORG   $000B                   /* Timer 0 interrupt     (X10 pulses)       */
   LJMP  Tmr0Tick
   ORG   PWRHERE
 }

/*--- global variables                                                       */

   ACBits0 = ACBits1 = 0;              /* flush all flags at once            */

   Cycles = 0;                         /* start counting upward              */
   XModState = XMOD_ADDRESS;           /* ready for addresses                */
   memset(ModSelect,0,sizeof(ModSelect));
   memset(ModStatus,XSTATUS_UNKNOWN,sizeof(ModStatus));

   T0State = 0;                        /* preset for first interrupt         */
   T0Ptr = T0Times;
   T0Next = T0Times[T0State];

   PwrState = PWR_GOOD;                /* assume this is true...             */
   GoodCyc = BadCyc = 0;
   SETBIT(POWERFLAG);                  /* indicate power is OK               */

   _memset(XOutBits,0,sizeof(XOutBits));           /* flush bit buffer       */
   memset(XTrnBuff,0,sizeof(XTrnBuff));            /* flush outgoing buffer  */
   XOutHead = XOutTail = XOutCtr = XOutMax = 0;    /* indicate no messages   */
   XOutPtr = XTrnBuff;                             /* point to first element */

   memset(XRecBuff,0,sizeof(XRecBuff));            /* flush incoming buffer  */
   XInState = XIN_IDLE;                            /* nothing going down     */
   XInHead = XInTail = XInCtr = XInMax = 0;        /* indicate no messages   */

   RmtCtlMode = 1;                     /* update tables from remote controls */

/*--- Set up Timer 0 for long-period timings                                 */
/*    First AC line interrupt will handle synchronization                    */

   TCON &= 0xCF;                 /* clear run & interrupt flags              */

   TMOD &= 0xF0;                 /* set mode 1 (16-bit reload)               */
   TMOD |= 0x01;

   TH0 = TL0 = 0;                /* ensure line interrupt occurs first!      */

   SETBIT(PT0);                  /* high priority                            */
   SETBIT(ET0);                  /* interrupt enabled                        */
   SETBIT(TR0);                  /* start it running                         */

/*--- Set up External Interrupt 0 for edge-triggered mode                    */

   SETBIT(IT0);
   SETBIT(EX0);

}


/*---------------------------------------------------------------------------*/
/* Timer 0 interrupt handler                                                 */
/* This measures out times for X10 transmission & reception                  */
/* It's all in assembler because we need every cycle we can get...           */

Tmr0Tick() {

/*--- save regs                                                              */

;
 asm {
   SETB  TIMERFLAG
   PUSH  A
   PUSH  PSW

*--- send X10 data on the tick

   MOV   C,FlagNextTBit          ; fetch next X10 bit
   CPL   C                       ;  ... low = transmit, so invert it
   MOV   X10TRANS,C              ; send it to the transmitter

*--- reload the timer
*    the reload value is 16 bits, so we isolate each half

   CLR   TR0
   MOV   TL0,T0Next
   MOV   TH0,T0Next+1
   SETB  TR0


*--- handle X10 pulse timing
*    the state number is for the current interrupt

   PUSH  B
   PUSH  DPH
   PUSH  DPL
   PUSH  0                             ; R0 in disguise

   MOV   A,T0State                     ; set up index into table
   CJB   A,#NUMT0STATES,T0OK           ; ensure we have good index
   ORL   ErrFlags,#ERR_T0STATE         ; fix it if bad!
   MOV   DPTR,#T0Times
   MOV   T0Ptr,DPL
   MOV   T0Ptr+1,DPH
   MOVX  A,[DPTR]
   MOV   T0Next,A
   INC   DPTR
   MOVX  A,[DPTR]
   MOV   T0Next+1,A
   MOV   A,#0
   MOV   T0State,A

T0OK EQU *
   ADD   A,A                           ; (need word pointer)
   ADD   A,#LOW(Tmr0Fns)
   MOV   DPL,A
   MOV   A,#HIGH(Tmr0Fns)
   ADDC  A,#0
   MOV   DPH,A

   MOVX  A,[DPTR]                      ; high byte of function address
   MOV   B,A
   INC   DPTR
   MOVX  A,[DPTR]                      ; low byte (DW is high/low...)

   PUSH  A                             ; simulate CALL to do a jump
   PUSH  B

* set up next X10 data bit assuming it is the normal data output
*  the interrupt handlers shift the bits around to make this true...

   MOV   A,XOutBits
   MOV   C,A.7
   ANL   C,FlagSendX10                 ; disable output if not sending!
   MOV   FlagNextTBit,C

   RET                                 ; this is the indexed LJMP!

* branch table to get access to functions
* these must match the defines in the ACLINE.D2 file...

Tmr0Fns EQU *
   DW    X_1A_B
   DW    X_1A_C
   DW    X_1A_E
   DW    X_1B_B
   DW    X_1B_C
   DW    X_1B_E
   DW    X_1C_B
   DW    X_1C_C
   DW    X_1C_E
   DW    X_1_F
   DW    X_2A_B
   DW    X_2A_C
   DW    X_2A_E
   DW    X_2B_B
   DW    X_2B_C
   DW    X_2B_E
   DW    X_2C_B
   DW    X_2C_C
   DW    X_2C_E
   DW    X_2_F
   DW    X_D
   DW    0                       ; allow disassembler to re-sync
   DW    0

* X10 transmitter and receiver functions
* Remember that these are preparing for the _next_ interrupt...
*  the transmitter data for the current interrupt is already history
*  receiver data was sampled above and must be processed below

*--- first half cycle

X_1A_B   EQU *                   ; start of Phase A bit
   CPL   TIMERFLAG               ; send identification to scope
   CPL   TIMERFLAG
   MOV   C,FlagNewRBit           ; set up for next receiver sample
   MOV   FlagOldRBit,C
   LJMP  T0_sd

X_1A_C   EQU *                   ; center of Phase A bit
   MOV   C,X10REC                ; sample the input data
   CPL   C                       ; fix inverted bit
   MOV   FlagNewRBit,C           ; save for later
   CLR   FlagNextTBit            ; turn output off on next interrupt
   LJMP  T0_sd

X_1A_E   EQU *                   ; end of Phase A bit
   LJMP  T0_sd

X_1B_B   EQU *                   ; beginning of Phase B bit
   LJMP  T0_sd

X_1B_C   EQU *                   ; center of Phase B bit
   CLR   FlagNextTBit            ; turn output off on next interrupt
   LJMP  T0_sd

X_1B_E   EQU *                   ; end of Phase B bit
   LJMP  T0_sd

X_1C_B   EQU *                   ; beginning of Phase C bit
   LJMP  T0_sd

X_1C_C   EQU *                   ; center of Phase C bit
   CLR   FlagNextTBit            ; turn output off on next interrupt
   LJMP  T0_sd

X_1C_E   EQU *                   ; end of Phase C bit
   JNB   FlagSendX10,T0_1CE
   RL32  XOutBits                ; shift in next transmitter bit
   CLR   FlagNextTBit            ; but hold off for next interrupt
   MOV   A,XOutPos               ; handle odd bit counts...
   JZ    T0_1CE
   DEC   XOutPos                 ; account for this bit
T0_1CE   EQU *
   LJMP  T0_sd

*--- interrupt just before end of first half cycle
*    run receiver state machine to decode incoming bit status

X_1_F    EQU *                   ; final interrupt in first half cycle
   LJMP  XIcheck                 ; do the state machine two-step

*--- second half cycle

X_2A_B   EQU *                   ; start of Phase A bit
   MOV   C,FlagNewRBit           ; set up for next receiver sample
   MOV   FlagOldRBit,C
   LJMP  T0_sd

X_2A_C   EQU *                   ; center of Phase A bit
   MOV   C,X10REC                ; sample the input data
   CPL   C                       ; fix inverted bit
   MOV   FlagNewRBit,C           ; save for later
   CLR   FlagNextTBit            ; turn output off on next interrupt
   LJMP  T0_sd

X_2A_E   EQU *                   ; end of Phase A bit
   LJMP  T0_sd

X_2B_B   EQU *                   ; beginning of Phase B bit
   LJMP  T0_sd

X_2B_C   EQU *                   ; center of Phase B bit
   CLR   FlagNextTBit            ; turn output off on next interrupt
   LJMP  T0_sd

X_2B_E   EQU *                   ; end of Phase B bit
   LJMP  T0_sd

X_2C_B   EQU *                   ; beginning of Phase C bit
   LJMP  T0_sd

X_2C_C   EQU *                   ; center of Phase C bit
   CLR   FlagNextTBit            ; turn output off on next interrupt
   LJMP  T0_sd

X_2C_E   EQU *                   ; end of Phase C bit
   JNB   FlagSendX10,T0_2CE
   RL32  XOutBits                ; shift up next transmitter bit
   CLR   FlagNextTBit            ; but hold output off for next interrupt
   DEC   XOutPos                 ; account for this bit
T0_2CE   EQU *
   LJMP  T0_sd

*--- interrupt just before end of second half cycle
*    handle transmitter message repeat, startup, or shutdown

X_2_F    EQU *                   ; final interrupt in second half-cycle

   JB    FlagSendX10,XOsend      ; send in progress?

*  not currently sending a message, is one pending?

   MOV   A,XOutCtr               ; anything in the ring for us?
   LJZ   XOdone                  ; zero, so punt

   MOV   DPL,XOutPtr             ; nonzero, so msg is ready!
   MOV   DPH,XOutPtr+1           ; get ring pointer

   LJMP  XOsetup                 ; and prepare to send new message!


* currently sending a message, so check for doneness

XOsend   EQU *

   MOV   A,XOutPos               ; done shifting all bits?
   LJNZ  XOdone                  ; nonzero means keep rolling

   MOV   DPL,XOutPtr             ; point to ring buffer again
   MOV   DPH,XOutPtr+1
   MOVX  A,[DPTR]                ; get repeat count

   ORL   ErrFlags,#ERR_BADEMPTY  ; flag potential error
   JZ    XOnext                  ; this should not be possible...
   ANL   ErrFlags,#~ERR_BADEMPTY ; clear flag, no error found
   DEC   A                       ;  ... unless ring setup is bad
   MOVX  [DPTR],A                ; save counter for next pass
   JNZ   XOsetup                 ; nonzero says set message up again

* done with message, advance to next ring buffer entry

XOnext   EQU *                   ; done with this message, check for more

   MOV   A,XOutTail              ; move to next ring entry
   INC   A
   CJB   A,#XTRANSBUFFLEN,XOnext1
   CLR   A
XOnext1  EQU *
   MOV   XOutTail,A

   MOV   B,#XTRANSMSGLEN         ; convert index to offset
   MUL
   ADD   A,#LOW(XTrnBuff)
   MOV   DPL,A
   MOV   A,B
   ADDC  A,#HIGH(XTrnBuff)
   MOV   DPH,A

   MOV   XOutPtr,DPL             ; save new entry pointer
   MOV   XOutPtr+1,DPH

   DEC   XOutCtr                 ; is this entry empty?
   MOV   A,XOutCtr
   JNZ   XOsetup                 ; nonzero means something to do

* nothing more to do, so disable transmissions

XOstop   EQU *
   CLR   FlagSendX10             ; really done, disable transmissions
   CLR   FlagNextTBit            ; and send no more data
   SETB  X10MSG                  ; shut off outside indicator
   LJMP  XOdone

* set up a new message...
* DPTR is aimed at the ring buffer entry
* Does _not_ validate the rep count, so you get one rep for zero counts

XOsetup  EQU *

   INC   DPTR                    ; step over repeat counter
   MOVX  A,[DPTR]
   MOV   XOutPos,A               ; set up message length

   INC   DPTR                    ; point to message bits
   MOV   B,#XRAWBUFFLEN          ; set byte counter
   MOV   R0,#XOutBits            ; to shift buffer
XOset1   EQU *
   MOVX  A,[DPTR]
   MOV   [R0],A
   INC   DPTR
   INC   R0
   DJNZ  B,XOset1

   MOV   A,XOutBits              ; preset first data bit
   MOV   C,A.7
   MOV   FlagNextTBit,C

   SETB  FlagSendX10             ; enable transmission

   CLR   X10MSG                  ; indicate transmission starts now

   CPL   X10MSG                  ; scope sync (if off before)
   NOP
   CPL   X10MSG

XOdone   EQU *
   LJMP  XIcheck                 ; do the receiver state machine two-step

*--- deadman interrupt that occurs only if AC line sync fails

X_D EQU *
   CPL   TIMERFLAG                     ; obvious scope sync
   MOV   T0State,#0                    ; force reset to first state
   MOV   DPTR,#T0Times                 ; set up timer pointer
   MOV   T0Ptr,DPL
   MOV   T0Ptr+1,DPH
   CPL   TIMERFLAG                     ; obvious scope sync
   CPL   TIMERFLAG                     ; obvious scope sync
   SETB  FlagDeadMan                   ; indicate this is a faked interrupt
   SETB  IE0                           ; takes effect after return
   CPL   TIMERFLAG
   LJMP  T0_dm                       ; skip normal setup...


*--- X10 receiver state machine
*    this is executed at the end of each half cycle to decode incoming bits
*    the FlagSkipHalf flag tells us we got a data bit in the last cycle
*     so we need to wait a full cycle before the next one

XIcheck  EQU *

   LJBC  FlagSkipHalf,T0_In9           ; skip & resync next half cycle

   MOV   A,XInState                    ; do indexed jump through table
   CJBE  A,#XIN_D16,T0_In1             ; ensure good index
   MOV   A,#XIN_D16+1                  ; punt if bad state
T0_In1 EQU *
   ADD   A,A                           ; SJMPs are two bytes each
   MOV   DPTR,#XInTable

   INC   XInState                      ; will be reset if invalid bit
   SETB  FlagSkipHalf                  ; ditto

   XRLB  FlagNewRBit,FlagOldRBit       ; set up XOR of two bits in C
   MOV   F0FLAG,C                      ;  ... and hide away for later
   MOV   C,FlagOldRBit                 ; set up "true" bit for shifting

   JMP   [A+DPTR]

XInTable EQU *
   SJMP  XInIdle
   SJMP  XInStart
   SJMP  XInH1
   SJMP  XInH2
   SJMP  XInH4
   SJMP  XInH8
   SJMP  XInD1
   SJMP  XInD2
   SJMP  XInD4
   SJMP  XInD8
   SJMP  XInD16
   SJMP  XInOops

XInIdle  EQU *
   JNB   FlagOldRBit,T0_In8      ; first two half-cycles of start are 11
   JNB   FlagNewRBit,T0_In8
   SJMP  T0_In9

XInStart EQU *
   JNB   FlagOldRBit,T0_In8            ; next two half-cycles must be 10
   JB    FlagNewRBit,T0_In8
   MOV   XInMsg,#0                     ; bingo... clear bits for new data
   MOV   XInMsg+1,#0
   SJMP  T0_In9

XInH1    EQU *
   JNB   F0FLAG,T0_In8                 ; from now on, bits must be different
   MOV   A,XInMsg+1
   RLC   A
   MOV   XInMsg+1,A
   SJMP  T0_In9

XInH2    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg+1
   RLC   A
   MOV   XInMsg+1,A
   SJMP  T0_In9

XInH4    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg+1
   RLC   A
   MOV   XInMsg+1,A
   SJMP  T0_In9

XInH8    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg+1
   RLC   A
   MOV   XInMsg+1,A
   SJMP  T0_In9

* these are here to allow SJMPs from the other routines

XInOops  EQU *                         ; invalid state...
   ORL   ErrFlags,#ERR_XINSTATE        ; record the glitch

T0_In8 EQU *                           ; bad bits or finished or some such
   MOV   XInState,#XIN_IDLE            ;  reset to starting state
   CLR   FlagSkipHalf                  ; re-scan current new bit next time
T0_In9 EQU *
   LJMP  T0_sd


XInD1    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg
   RLC   A
   MOV   XInMsg,A
   SJMP  T0_In9

XInD2    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg
   RLC   A
   MOV   XInMsg,A
   SJMP  T0_In9

XInD4    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg
   RLC   A
   MOV   XInMsg,A
   SJMP  T0_In9

XInD8    EQU *
   JNB   F0FLAG,T0_In8
   MOV   A,XInMsg
   RLC   A
   MOV   XInMsg,A
   SJMP  T0_In9

XInD16   EQU *                   ; final bit...
   JNB   F0FLAG,T0_In8

   MOV   A,XInMsg
   RLC   A
   MOV   XInMsg,A

   MOV   DPTR,#XRecBuff                ; tuck into External RAM ring buffer
   MOV   A,XInHead                     ;  get word offset
   ADD   A,A
   AddDPTR A
   MOV   A,XInMsg                      ; do low byte
   MOVX  [DPTR],A
   INC   DPTR
   MOV   A,XInMsg+1                    ; and high byte
   MOVX  [DPTR],A

   INC   XInCtr                      ; tick size counter
   MOV   A,XInCtr                    ; track maximum value
   CJBE  A,XInMax,XInD16a
   MOV   XInMax,A
XInD16a EQU *

   INC   XInHead                       ; tick array index
   MOV   A,XInHead                     ; track maximum value
   CJB   A,#XRECBUFFLEN,XInD16b
   MOV   XInHead,#0
XInD16b EQU *

   SJMP  T0_In8                      ; and resest state for next message


*--- set up for next interrupt

T0_sd  EQU *

   INC   T0State                       ; advance state index

   MOV   DPH,T0Ptr+1                   ; step timer reload pointer
   MOV   DPL,T0Ptr
   INC   DPTR
   INC   DPTR
   MOV   T0Ptr+1,DPH
   MOV   T0Ptr,DPL

T0_dm
   MOVX  A,[DPTR]                      ; set up timer reload value (low byte)
   MOV   T0Next,A
   INC   DPTR
   MOVX  A,[DPTR]                      ; fetch high byte
   MOV   T0Next+1,A


*--- restore regs and return...

   POP   0                       ; restore R0
   POP   DPL
   POP   DPH
   POP   B
   POP   PSW
   POP   A

   CLR   TIMERFLAG
   RETI                          ; pre-empt normal C level return

 }

}


/*---------------------------------------------------------------------------*/
/* External Interrupt 0 handler                                              */
/* This is triggered by high-to-low zero crossing input                      */
/* Remember that we only get one interruption per cycle, not two!            */
/* Sends out X10 bit, which should be zero if no transmission is in progress */

ExtInt0() {

/*--- save regs                                                              */

;
 asm {
   SETB  LINEFLAG
   PUSH  A
   PUSH  B
   PUSH  PSW
   PUSH  DPH
   PUSH  DPL


*--- send X10 bit to ensure correct timing after down-going zero crossing

   MOV   C,FlagNextTBit          ; fetch next X10 bit
   CPL   C                       ;  ... fix output inversion
   MOV   X10TRANS,C              ; send it to the transmitter

*--- force Timer 0 into sync with AC line input
*    the timer interrupt occurs _right away_ when the bit gets set...

   MOV   A,#0                          ; already in sync?
   CJNE  A,T0State,Int0_1              ; no, so do the force
   SJMP  Int0_2                        ; yes, bypass overhead

Int0_1   EQU *
   MOV   T0State,A                     ; force initial value
   MOV   DPTR,#T0Times                 ; set up reload value
   MOV   T0Ptr+1,DPH
   MOV   T0Ptr,DPL
   MOVX  A,[DPTR]
   MOV   T0Next,A
   INC   DPTR
   MOVX  A,[DPTR]
   MOV   T0Next+1,A

Int0_2   EQU *
   SETB  TF0                           ; immediate int to bypass deadman

*--- do power-fail checks

   MOV   A,PwrState                    ; do indexed jump through table
   CJBE  A,#PWR_DEAD,PwrIOK            ; ensure good index
   MOV   A,#PWR_DEAD+1
PwrIOK   EQU *
   ADD   A,A                           ; SJMPs are two bytes each
   MOV   DPTR,#PwrTable
   JMP   [A+DPTR]

PwrTable EQU *
   SJMP PwrGood
   SJMP PwrFail
   SJMP PwrDead
   SJMP PwrOops

* power was good up to this point

PwrGood  EQU *
   JB    FlagDeadMan,PwrGood1          ; missing cycle?
   LJMP  PwrDone                       ; nope, so still good

PwrGood1 EQU *                         ; first bad cycle in a while
   MOV   PwrState,#PWR_FAILING
   MOV   GoodCyc,#0                    ; restart from scratch
   MOV   BadCyc,#1                     ; count first bad cycle
   LJMP  PwrDone

* missed at least one cycle recently

PwrFail EQU *
   JB    FlagDeadMan,PwrFail1          ; missing cycle?
   INC   GoodCyc                       ; no, so count a good one
   MOV   BadCyc,#0                     ; and clear failure counter
   MOV   A,GoodCyc                     ; had enough?
   CJB   A,#LINE_DROPOUT,PwrDone       ; below -> not yet
   MOV   PwrState,#PWR_GOOD            ; yes, so back to normal
   LJMP  PwrDone

PwrFail1 EQU *
   MOV   GoodCyc,#0                    ; clear success counter
   INC   BadCyc                        ; count the missing cycle
   MOV   A,BadCyc                      ; had enough?
   CJB   A,#LINE_DROPOUT,PwrDone       ; below -> not yet
   SETB  FlagPwrFail                   ; yes, so hard failure
   SETB  FlagAnyPFail                  ; and remember until reset
   CLR   POWERFLAG                     ; turn on LED
   MOV   PwrState,#PWR_DEAD
   SJMP  PwrDone

* power is definitely bad

PwrDead  EQU *
   JB    FlagDeadMan,PwrDead1          ; missing cycle?
   MOV   BadCyc,#0                     ; no, clear bad cycle counter
   INC   GoodCyc                       ; count good cycle
   MOV   A,GoodCyc                     ; had enough?
   CJB   A,#LINE_DROPOUT,PwrDone       ; below -> not yet
   MOV   PwrState,#PWR_GOOD            ; yes, so try restarting
   CLR   FlagPwrFail
   SETB  POWERFLAG                     ; turn off LED
   SJMP  PwrDone

PwrDead1 EQU *
   MOV   GoodCyc,#0                    ; still dead, so no joy
   SJMP  PwrDone

* invalid index entry, patch it up

PwrOops  EQU *
   MOV   PwrState,#PWR_GOOD            ; force this true
   ORL   ErrFlags,#ERR_PWRSTATE        ; record the glitch
   SJMP  PwrDone

PwrDone EQU *
   CLR   FlagDeadMan
   CLR   IE0


*--- count off cycles and set once-per-second flag

   INC   Cycles                        ; count upward to max value
   MOV   A,Cycles
   CJB   A,#LINE_FREQ,PwrMid           ; are we done?
   MOV   Cycles,#0                     ; yes, so restart
   SETB  FlagSecond                    ;  ... leave flag of this event
PwrMid   EQU *

   POP   DPL
   POP   DPH
   POP   PSW
   POP   B
   POP   A
   CLR   LINEFLAG
   RETI                          ; pre-empt normal return
 }

}



/*---------------------------------------------------------------------------*/
/* Display array indexes as useful information...                            */
/* Inputs are array indexes, not receiver bit patterns                       */

PwrShowHBits(HIndex)
WORD HIndex;
{
   sprintf(CmdRespBuff,"%c=%02x",
           HIndex+'A',HouseBits[HIndex][HOUSERECPATTERN]);
   Nputstr(CmdRespBuff);
}

PwrShowDFBits(DFIndex)
WORD DFIndex;
{
   sprintf(CmdRespBuff,"%02x",FuncBits[DFIndex][FUNCRECPATTERN]);
   Nputstr(CmdRespBuff);
}

PwrShowDFName(DFIndex)
WORD DFIndex;
{
   sprintf(CmdRespBuff,"%s",DevFuncStrings[DFIndex]);
   Nputstr(CmdRespBuff);
}

/*---------------------------------------------------------------------------*/
/* Convert from receiver bit patterns to array indexes                       */

WORD PwrCvtHRec(HRec)
WORD HRec;
{
   return HouseRecLetters[HRec] - 'A';
}

WORD PwrCvtDFRec(DFRec)
WORD DFRec;
{
   return FuncRecIndex[DFRec];
}


/*---------------------------------------------------------------------------*/
/* Return oldest message from X10 receiver queue                             */
/* Returns 0xFFFF if no message available                                    */

WORD PwrGetMsg() {
WORD Msg;

   if (XInCtr) {
      Msg = XRecBuff[XInTail];
      ++XInTail;
      if (XRECBUFFLEN <= XInTail) {
         XInTail = 0;
      }
      --XInCtr;
      if (0 != (ReportMode & REPORT_REC)) {
         Nputstr("Rcv:  H=");
         PwrShowHBits(PwrCvtHRec((Msg & XMSG_HBITS) >> XMSG_HSHIFT));
         Nputstr(" D/F=");
         PwrShowDFBits(PwrCvtDFRec((Msg & XMSG_FBITS) >> XMSG_FSHIFT));
         Nputstr(" (");
         PwrShowDFName(PwrCvtDFRec((Msg & XMSG_FBITS) >> XMSG_FSHIFT));
         Nputstr(")\n");
      }
   }
   else {
      Msg = XMSG_NOMSG;
   }


   return Msg;

}


/*---------------------------------------------------------------------------*/
/* Flush receiver ring buffer                                                */

PwrFlushRec() {

   while (0xFFFF != PwrGetMsg()) {
      continue;
   }

}

/*---------------------------------------------------------------------------*/
/* Flush transmitter ring buffer                                             */

PwrFlushTrans() {

   CLRBIT(ET0);
   CLRBIT(EX0);

   CLRBIT(FlagNextTBit);
   CLRBIT(FlagSendX10);

   _memset(XOutBits,0,sizeof(XOutBits));
   memset(XTrnBuff,0,sizeof(XTrnBuff));
   XOutHead = XOutTail = XOutCtr = XOutMax = 0;
   XOutPtr = XTrnBuff;
   SETBIT(X10MSG);

   SETBIT(EX0);
   SETBIT(ET0);


}

/*---------------------------------------------------------------------------*/
/* Update module select & status arrays based on message content             */
/* Msg is in receiver bit format, switches are in internal format            */
/* Clears all other housecodes for each address, thus avoiding the           */
/*  conflict with interleaved messages:  A-1 B-1 B-ON A-ON affects only B-1  */

PwrUpdateState(Msg)
WORD Msg;
{
BYTE FnIndex;
BYTE HIndex;
BYTE HClr;

   HIndex =  HouseRecLetters[(Msg & XMSG_HBITS) >> XMSG_HSHIFT] - 'A';
   FnIndex = FuncRecIndex[(Msg & XMSG_FBITS) >> XMSG_FSHIFT];

   switch (XModState) {
   case XMOD_ADDRESS :
      if (16 > FnIndex) {                          /* if device address      */
         for (HClr = 0; HClr < 16; ++HClr) {       /* clear other housecodes */
            if (HClr != HIndex) {                  /*  not this one!         */
               memset(ModSelect[HClr],0,sizeof(ModSelect[0]));
            }
         }
         ModSelect[HIndex][FnIndex] = 1;           /* other in house OK...   */
      }
      else {                                       /* if command             */
         PwrSimCmd(HIndex,FnIndex);                /*  ... simulate it       */
         XModState = XMOD_COMMAND;                 /* and enter command mode */
#ifdef PWR_TRACE
         Nputstr("  to command mode\n");
#endif
      }
      break;
   case XMOD_COMMAND :
      if (16 > FnIndex) {                          /* if device address      */
         memset(ModSelect,0,sizeof(ModSelect));    /*  ... deselect others   */
         ModSelect[HIndex][FnIndex] = 1;           /*  ... select it         */
         XModState = XMOD_ADDRESS;                 /* and enter address mode */
#ifdef PWR_TRACE
         Nputstr("  to address mode\n");
#endif
      }
      else {                                       /* if command             */
         PwrSimCmd(HIndex,FnIndex);                /*  ... simulate it       */
      }
      break;
   default :                                       /* this cannot happen...  */
      ErrFlags |= ERR_XCMDSTATE;                   /* so record it for later */
      XModState = XMOD_ADDRESS;                    /* and reinitialize       */
      memset(ModSelect,0,sizeof(ModSelect));
      memset(ModStatus,XSTATUS_UNKNOWN,sizeof(ModStatus));
   }

}


/*---------------------------------------------------------------------------*/
/* Simulate effect of command on status arrays                               */
/* Parameters are internal indexes                                           */
/* For broadside on/off commands, change only individually addressed modules */
/*  (this avoids changing the state of modules that don't exist)             */
/* Adds ON/OFF commands to refresh list                                      */
/* Affects only selected modules with same housecode                         */

PwrSimCmd(HIndex,FnIndex)
WORD HIndex;
WORD FnIndex;
{

BYTE ModIndex;

   switch (FnIndex) {
   case FNDX_ON :                                  /* this gets refreshed    */
      for (ModIndex=0; ModIndex<16; ++ModIndex) {
         if (ModSelect[HIndex][ModIndex]) {
            ModStatus[HIndex][ModIndex] = XSTATUS_ON;
            RefNewEvent(HIndex,ModIndex,FnIndex,2);
         }
      }
      break;
   case FNDX_DIM :
   case FNDX_BRIGHT :
      for (ModIndex=0; ModIndex<16; ++ModIndex) {
         if (ModSelect[HIndex][ModIndex]) {
            ModStatus[HIndex][ModIndex] = XSTATUS_ON;       /* device ON     */
            RefNewEvent(HIndex,ModIndex,FnIndex,0);         /* clr ref event */
         }
      }
      break;
   case FNDX_OFF :                                 /* this gets refreshed    */
      for (ModIndex=0; ModIndex<16; ++ModIndex) {
         if (ModSelect[HIndex][ModIndex]) {
            ModStatus[HIndex][ModIndex] = XSTATUS_OFF;
            RefNewEvent(HIndex,ModIndex,FnIndex,2);
         }
      }
      break;
   case FNDX_ALLOFF :                              /* for single housecode   */
      RefNewEvent(HIndex,ModIndex,FnIndex,0);      /* clear refresh events   */
      for (ModIndex=0; ModIndex<16; ++ModIndex) {
         if (XSTATUS_UNKNOWN != ModStatus[HIndex][ModIndex]) {
            ModStatus[HIndex][ModIndex] = XSTATUS_OFF;
         }
      }
      memset(ModSelect,0,sizeof(ModSelect));       /* all are unselected now */
      break;
   default :
      break;
   }

}


/*---------------------------------------------------------------------------*/
/* Show current state of modules for given house code                        */
/* If house number is 0xFF, dumps all modules for all house codes            */
/*                    0xFE, dumps all modules in hex format                  */
/* If module number is 0xFF, shows all modules for given house code          */

PwrShowModule(HIndex,MIndex)
WORD HIndex;
WORD MIndex;
{

WORD HState;
WORD MState;

   switch (HIndex) {
   case 0xFF :                                     /* dump all house codes   */
      for (HIndex=0; HIndex<16; ++HIndex) {
         PwrShowModule(HIndex,0xFF);
      }
      break;
   case 0xFE :                                     /* dump in compressed fmt */
      for (HIndex=0; HIndex<16; ++HIndex) {
         HState = 0;
         for (MIndex=0; MIndex<16; ++MIndex) {
            MState = (XSTATUS_ON == ModStatus[HIndex][MIndex]) ? 1 : 0;
            HState |= MState << MIndex;
         }
         sprintf(CmdRespBuff,"%04x",HState);
         Nputstr(CmdRespBuff);
      }
      Nputch('\n');
      break;
   default :
      if (0xFF == MIndex) {                        /* dump all modules       */
         sprintf(CmdRespBuff,"%c=",HIndex+'A');
         Nputstr(CmdRespBuff);
         for (MIndex=0; MIndex<16; ++MIndex) {
            Nputch(XStatusChar[ModStatus[HIndex][MIndex]]);
         }
         Nputch('\n');
      }
      else {                                       /* dump single module     */
         sprintf(CmdRespBuff,"%c%02u=%s\n",
                HIndex+'A',
                MIndex+1,
                XStatusMsg[ModStatus[HIndex][MIndex]]);
         Nputstr(CmdRespBuff);
      }
   }

}


/*---------------------------------------------------------------------------*/
/* Convert house and function codes to transmitter bit format                */
/* Output is formatted for the transmitter ring buffer                       */
/* Does not set bit length or number of repeats for this message             */
/* House, device, function are all indexes into the ACLINE tables            */

PwrMakeTBits(House,DevFn,pBuffer)
WORD House;
WORD DevFn;
BYTE *pBuffer;
{

unsigned char ByteIndex;

/*--- clear message and set start/end bits                                   */

   move(pBuffer+XMSG_TBITS,StartBits,XRAWBUFFLEN);

/*--- set up housecode                                                       */

   for (ByteIndex=0; ByteIndex<XRAWBUFFLEN; ByteIndex++) {
      *(pBuffer+XMSG_TBITS + ByteIndex) |= HouseBits[House][ByteIndex];
   }

/*--- set up function code                                                   */

   for (ByteIndex=0; ByteIndex<XRAWBUFFLEN; ByteIndex++) {
      *(pBuffer+XMSG_TBITS + ByteIndex) |= FuncBits[DevFn][ByteIndex];
   }

}


/*---------------------------------------------------------------------------*/
/* Create message bits and tuck into transmitter ring                        */
/* Output will start automagically when XOutCtr becomes nonzero              */
/* House, device, function are all indexes into the ACLINE tables            */

PwrFormatMsg(House,DevFn,BitLen,Repeats)
WORD House,
WORD DevFn;
unsigned int BitLen;
unsigned int Repeats;
{
BYTE Temp;

   if (XTRANSBUFFLEN <= XOutCtr) {                 /* check for queue room   */
      ErrFlags |= ERR_TQFULL;                      /* flag full event        */
      while (XTRANSBUFFLEN <= XOutCtr);            /* wait for space avail   */
   }

   PwrMakeTBits(House,DevFn,&(XTrnBuff[XOutHead][0]));
   XTrnBuff[XOutHead][XMSG_REPEATS] = Repeats;
   XTrnBuff[XOutHead][XMSG_BITLEN]  = BitLen;

#ifdef PWR_TRACE
   sprintf(CmdRespBuff,"  Formatted output at slot %u is",XOutHead);
   Nputstr(CmdRespBuff);
   for (Temp=0; Temp<XTRANSMSGLEN; ++Temp) {
      sprintf(CmdRespBuff," %02x",XTrnBuff[XOutHead][Temp]);
      Nputstr(CmdRespBuff);
   }
   Nputstr("\n");
#endif

   ++XOutHead;                         /* tick input index                   */
   if (XTRANSBUFFLEN <= XOutHead) {
      XOutHead = 0;
   }

   Temp = ++XOutCtr;                   /* tick level & trigger transmission  */
   if (XOutMax < Temp) {               /*  ++XOutCtr is really atomic!       */
      XOutMax = Temp;
   }

}


/*---------------------------------------------------------------------------*/
/* Create X10 message given all the information...                           */
/* This produces several messages with varying repeats for each part...      */
/* House, device, function are all indexes into the ACLINE tables            */

PwrSendSeq(House,Device,Func,Repeats)
WORD House,
WORD Device,
WORD Func;
unsigned int Repeats;
{

/*--- dump this message in readable format                                   */

   if (0 != (ReportMode & REPORT_TRANS)) {
      Nputstr("EnQ:  H=");
      PwrShowHBits(House);
      Nputstr(" D=");
      PwrShowDFBits(Device);
      Nputstr(" F=");
      PwrShowDFBits(Func);
      sprintf(CmdRespBuff," Rep=%-2u (",Repeats);
      Nputstr(CmdRespBuff);
      PwrShowDFName(Device);
      Nputch(' ');
      PwrShowDFName(Func);
      Nputstr(")\n");
   }

   PwrFormatMsg(House,Device,XTMSGSHORT,1);        /* no pause after msg     */
   PwrFormatMsg(House,Device,XTMSGBITS,1);         /* short pause            */

   Repeats = max(1,Repeats);                       /* ensure good repeats    */
   Repeats = min(255,Repeats);

   if (1 == Repeats) {
      PwrFormatMsg(House,Func,XTMSGBITS,1);        /* ensure pause           */
   }
   else {
      PwrFormatMsg(House,Func,XTMSGSHORT,Repeats-1);        /* no pauses     */
      PwrFormatMsg(House,Func,XTMSGBITS,1);        /* short pause here       */
   }


}



/*---------------------------------------------------------------------------*/
/* Return array index corresponding to ASCII function code                   */
/* Returns -1 (FFFF) if code isn't in the table                              */

int PwrFindFunc(pCode)
char *pCode;
{
BYTE Index;
int  RetVal;

#ifdef PWR_TRACE
   printf("Searching for [%s]...",pCode);
#endif

   RetVal = -1;

   for (Index=0; Index<NUMFUNCNAMES; Index++) {
      if ((FuncNames[Index][0] == *pCode) &&
          (FuncNames[Index][1] == *(pCode+1))) {
         RetVal = Index;
         break;
      }
   }

#ifdef PWR_TRACE
   printf("  returning %02x\n",RetVal);
#endif
   return RetVal;

}

/*---------------------------------------------------------------------------*/
/* Dump current power state                                                  */
/* Side effect resets the maximum values of the ring buffer counters         */

PwrDumpState() {
unsigned char Index;

   Nputstr("Power line status:\n");

   sprintf(CmdRespBuff," Trans:  Head %02x, Tail %02x, Ctr %02x, ",
          XOutHead,XOutTail,XOutCtr);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"Max %02x, OutPtr %04x, ErrFlags %02x\n",
          XOutMax,XOutPtr,ErrFlags);
   ErrFlags = 0;
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"  [XOutHead] -> ");
   Nputstr(CmdRespBuff);
   for (Index=0; Index<XTRANSMSGLEN; Index++) {
      sprintf(CmdRespBuff," %02x",XTrnBuff[XOutHead][Index]);
      Nputstr(CmdRespBuff);
   }
   sprintf(CmdRespBuff,"\n  [XOutTail] -> ");
   Nputstr(CmdRespBuff);
   for (Index=0; Index<XTRANSMSGLEN; Index++) {
      sprintf(CmdRespBuff," %02x",XTrnBuff[XOutTail][Index]);
      Nputstr(CmdRespBuff);
   }
   sprintf(CmdRespBuff,"\n  [XOutPtr]  -> ");
   Nputstr(CmdRespBuff);
   for (Index=0; Index<XTRANSMSGLEN; Index++) {
      sprintf(CmdRespBuff," %02x",*(BYTE *)(XOutPtr + Index));
      Nputstr(CmdRespBuff);
   }
   sprintf(CmdRespBuff,"\n  XOutBits[] -> ");
   Nputstr(CmdRespBuff);
   for (Index=0; Index<XRAWBUFFLEN; Index++) {
      sprintf(CmdRespBuff," %02x",XOutBits[Index]);
      Nputstr(CmdRespBuff);
   }
   Nputch('\n');

   sprintf(CmdRespBuff," Rec:    Head %02x, Tail %02x, Ctr %02x, Max %02x\n",
          XInHead,XInTail,XInCtr,XInMax);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"  [XInHead] -> %04x",XRecBuff[XInHead]);
   Nputstr(CmdRespBuff);
   sprintf(CmdRespBuff,"  [XInTail] -> %04x",XRecBuff[XInTail]);
   Nputstr(CmdRespBuff);
   Nputch('\n');

   sprintf(CmdRespBuff," Powerfail:  ACBits %02x %02x, state %u, cyc %u, good %u, bad %u\n",
          ACBits0,ACBits1,PwrState,Cycles,GoodCyc,BadCyc);
   Nputstr(CmdRespBuff);

   XInMax = XOutMax = 0;

}

