/* ACLINE.H -- power line interface                                          */

/*---------------------------------------------------------------------------*/
/* Galactic power line interface constants                                   */

/*--- I/O bit locations                                                      */

#define ZEROCROSS    INT0
#define X10REC       T1
#define X10TRANS     INT1

/*--- AC line sync values                                                    */

#ifdef LINEFREQ60
#define LINE_FREQ    60                /* AC powerline frequency cycles/sec  */
#endif

#ifdef LINEFREQ50
#define LINE_FREQ    50                /* AC powerline frequency cycles/sec  */
#endif

#define LINE_DROPOUT 30                /* cycles before/after power fails    */

#define PWR_GOOD     0                 /* AC line sync is OK                 */
#define PWR_FAILING  1                 /* some bad cycles                    */
#define PWR_DEAD     2                 /* completely missing                 */


/*--- X10 receiver states                                                    */
/*    these are driven by the data sampled at the timer ticks                */
/*    must match the branch table in ACLINE.C...                             */

#define XIN_IDLE        0              /* no message in progress             */
#define XIN_D16         10             /*  ... D16                           */


/*--- layout of X10 receiver and internal format messages                    */
/*    each bit is a logical value and the start bits aren't included         */
/*    house & function codes are as defined in the X10 doc                   */

#define XMSG_HBITS   0x0F00            /* house code location                */
#define XMSG_HSHIFT  8                 /*  right shift required to align     */

#define XMSG_FBITS   0x001F            /* function code location             */
#define XMSG_FSHIFT  0                 /*  right shift required to align     */

#define XMSG_NOMSG   0xFFFF            /* guaranteed invalid message         */

#define XRECBUFFLEN  32                /* incoming message ring buffer size  */

/*--- layout of X10 transmitter ring buffer format messages                  */
/*    each bit is a half-cycle of power line output                          */
/*    start bits are on the left (first byte), stop bits on the right        */
/*    NOTE -- you get 4 ring buffer entries for each message!                */
/*            (length must be <256 entries for the assembler code)           */

#define XRAWBUFFLEN     4              /* bytes in outgoing message buffer   */
#define XTRANSMSGLEN    6              /* bytes in ring buffer               */
#define XTRANSBUFFLEN   254            /* outgoing message ring buffer size  */

/* these offsets are hardcoded into the interrupt routine, so be careful     */

#define XMSG_REPEATS 0                 /* offset of repeat count for message */
#define XMSG_BITLEN  1                 /* offset of message bit length       */
#define XMSG_TBITS   2                 /* offset of actual message bits      */

#ifdef x0
#define XTMSGBITS    28                /* 2 * (2 + 4 + 5 + 3) half cycles    */
#else
#define XTMSGBITS    32                /* 2 * (2 + 4 + 5 + 5) half cycles    */
#endif

#define XTMSGSHORT   22                /* 2 * (2 + 4 + 5) half cycles        */

