/* PLLINK.H -- fundamental constants                                         */

/*---------------------------------------------------------------------------*/
/* Power line frequency                                                      */
/* Remove the xxx suffix from the frequency you want...                      */
/* This is a touch clumsy, but Micro-C does not have a real #if parser       */

#define LINEFREQ60
#define LINEFREQ50xxx

#ifdef LINEFREQ50
#define FREQMSG      "50 Hz"
#endif

#ifdef LINEFREQ60
#define FREQMSG      "60 Hz"
#endif

/*---------------------------------------------------------------------------*/
/* Debugging outputs                                                         */

#define MAINFLAG   P1.7                /* heartbeat -- one change/second     */
#define X10MSG     P1.6                /* low during X10 transmissions       */
#define POWERFLAG  P1.5                /* goes on (low) for power failures   */
#define TIMERFLAG  P1.4                /* high for timer ticks               */
#define LINEFLAG   P1.3                /* high for AC line interrupts        */
#define SERIALFLAG P1.2                /* high for serial port interrupts    */
#define TRACEFLAG  P1.1                /* general tracing output             */
#define MANUALMODE P1.0                /* INPUT low -> manual mode on reset  */

#define TRACECHAR(x) putch(x)          /* how to output tracing stuff        */

#ifndef LOGGGING                       /* define LOGGING to enable outputs   */
#define TRACECHAR(x)
#endif


/*---------------------------------------------------------------------------*/
/* Bit masks for ErrFlags to monitor internal glitches                       */
/* They're in either assembler or C syntax as needed...                      */

#define ERR_PWRSTATE    $01            /* invalid power state variable       */
#define ERR_T0STATE     $02            /* bad T0State variable               */
#define ERR_XINSTATE    $04            /* bad X10 input state                */
#define ERR_XCMDSTATE   0x08           /* bad module cmd/addr state          */

#define ERR_BADEMPTY    16             /* counter zero on decrement          */
#define ERR_TQFULL      32             /* transmitter queue was full         */

/*---------------------------------------------------------------------------*/
/* Enables for various reporting outputs                                     */

#define REPORT_NONE     0x00           /* disable all reports                */
#define REPORT_REC      0x01           /* show receiever messages            */
#define REPORT_TRANS    0x02           /* show transmitter messages          */
#define REPORT_REFRESH  0x04           /* show refresh events                */

/*---------------------------------------------------------------------------*/
/* Stack initialization code                                                 */

#define MAXSTACK        127            /* last valid IRAM address for stack  */
                                       /* decimal for C/asm use...           */
