/* ANSI command decoder                                                      */


/*---------------------------------------------------------------------------*/
/* Initialize the ANSI decoder                                               */

ANSIInit(void) {

   ANSIState = ANSI_IDLE;
   ANSIParamCtr = 0;
   ANSISavedRow = ANSISavedCol = 1;

}

/*---------------------------------------------------------------------------*/
/* Put a character on the screen                                             */
/*  ... paying attention to the ANSI control sequences embedded therein...   */

ANSIPutChar(CharCode)
BYTE CharCode;
{

   switch (ANSIState) {

   case ANSI_IDLE:                     /* waiting for initial ESC char       */
      if (ESC == CharCode) {                       /* bingo!                 */
         ANSIState = ANSI_ESC;
         for (ANSIParamCtr = 0;                    /* set default numbers    */
              ANSIParamCtr < MAX_ANSI_PARAMS;
              ++ANSIParamCtr) {
            ANSIParams[ANSIParamCtr] = 1;
         }
         ANSIParamCtr = 0;
      }
      else {
         CGenPutChar(CharCode);        /* all other chars fall out here      */
      }
      break;

   case ANSI_ESC:                      /* expecting an open bracket          */
      if ('[' == CharCode) {
         ANSIState = ANSI_PARAM;
      }
      else {
         ANSIInit();
      }
      break;

   case ANSI_PARAM:                    /* working through parameters         */
      if ('=' != CharCode) {           /* discard this anomaly...            */
         if (isdigit(CharCode)) {
            ANSIParams[ANSIParamCtr] = CharCode - '0';
            ANSIState = ANSI_NUM;
         }
         else {
            if (isalpha(CharCode)) {
               ANSIDoCmd(CharCode);
            }
            else {
               if (';' == CharCode) {  /* semicolon without previous digit   */
                  ANSIParamCtr = min(++ANSIParamCtr,MAX_ANSI_PARAMS-1);
                  ANSIState = ANSI_PARAM;
               }
               else {
                  ANSIInit();
               }
            }
         }
      }
      break;

   case ANSI_NUM:                      /* accumulating a number              */
      if (isdigit(CharCode)) {
         ANSIParams[ANSIParamCtr] *= 10;
         ANSIParams[ANSIParamCtr] += CharCode - '0';
      }
      else {
         if (isalpha(CharCode)) {
            ANSIDoCmd(CharCode);
         }
         else {
            if (';' == CharCode) {
               ANSIParamCtr = min(++ANSIParamCtr,MAX_ANSI_PARAMS-1);
               ANSIState = ANSI_PARAM;
            }
            else {
               ANSIInit();
            }
         }
      }
      break;

   default:
      ANSIInit();
   }


}




/*---------------------------------------------------------------------------*/
/* Carry out an ANSI command code                                            */
/* We reach directly into the char gen variables to avoid trivial functions  */

ANSIDoCmd(CharCode)
BYTE CharCode;
{
BYTE Temp;

   ANSIState = ANSI_IDLE;              /* start searching for next sequence  */

   switch (CharCode) {
   case 'A':                           /* cursor up                          */
      asm {
      MOV   A,CGCurrentRow
      CLR   C
      SUBB  A,ANSIParams
      CJNE  A,CGNumCharRows,ADCA_0     /*  C set if new row < max            */
ADCA_0
      JC    ADCA_end                   /*  C clear if new row >= max         */
      CLR   A                          /*   so we stick at top margin        */
ADCA_end
      MOV   CGCurrentRow,A
      }
      break;
   case 'B':                           /* cursor down                        */
      CGCurrentRow = (CGCurrentRow + *ANSIParams) % CGNumCharRows;
      break;
   case 'C':                           /* cursor forward                     */
      CGCurrentCol = min(CGCurrentCol + *ANSIParams, CGNumCharCols - 1);
      break;
   case 'D':                           /* cursor backward                    */
      asm {
      MOV   A,CGCurrentCol
      CLR   C
      SUBB  A,ANSIParams
      CJNE  A,CGNumCharCols,ADCD_0     /*  C set if new col < max            */
ADCD_0
      JC    ADCD_end                   /*  C clear if new col >= max         */
      CLR   A                          /*   so we stick at left margin       */
ADCD_end
      MOV   CGCurrentCol,A
      }
      break;
   case 'H':                           /* set cursor position                */
   case 'f':
   case 'j':
      CGCurrentRow = min((*ANSIParams)-1,CGNumCharRows-1);
      CGCurrentCol = min(ANSIParams[1]-1,CGNumCharCols-1);
      break;
   case 'J':                           /* clear and home cursor              */
      if (2 == *ANSIParams) {
         CGenClear();
      }
      break;
   case 'K':                           /* erase EOL                          */
      for (Temp = CGCurrentCol; Temp < CGNumCharCols; ++Temp) {
         CGenWriteGlyph(' ',CGCurrentRow,Temp);
      }
      break;
   case 'h':                           /* set display mode                   */
      asm {
      MOV   A,ANSIParams
      JNZ   ADCh_n0                    /* 0 = large chars                    */
      MOV   A,#CG_LARGE
      SJMP  ADCh_do
ADCh_n0
      CJNE  A,#2,ADCh_n2               /* 2 = small chars                    */
      MOV   A,#CG_SMALL
ADCh_do
      PUSH  A
      PUSH  A
      LCALL CGenSize
      DEC   SP
      DEC   SP
      SJMP  ADCh_end
ADCh_n2
      CJNE  A,#7,ADCh_end              /* 7 = wrap at line end               */
      SETB  B_LineWrap
ADCh_end
      }
      break;
   case 'l':                           /* set video mode (restricted)        */
      asm {
      MOV   A,ANSIParams
      CJNE  A,#7,ADCl_end              /* 7 = reset wrap                     */
      CLR      B_LineWrap
ADCl_end
      }
      break;
   case 'm':                           /* set graphics rendition             */
      asm {
      MOV   A,ANSIParams
      JNZ   ADCm_n0                    /* 0 = normal attributes              */
      MOV   CGHighIntensity,#CG_NORMAL
      MOV   CGLowIntensity,#CG_OFF
      SJMP  ADCm_end
ADCm_n0
      CJNE  A,#1,ADCm_n1               /* 1 = bright                         */
      MOV   CGHighIntensity,#CG_BRIGHT
      MOV   CGLowIntensity,#CG_OFF
      SJMP  ADCm_end
ADCm_n1
      CJNE  A,#7,ADCm_end              /* 7 = reverse (swap current attrib)  */
      MOV   A,CGHighIntensity
      MOV   B,CGLowIntensity
      CJNE  A,B,ADCm_test0             /*   C set if High < Low = reversed   */
ADCm_test0
      JC    ADCm_end                   /*    (already reversed, do nothing)  */
      MOV   CGHighIntensity,B
      MOV   CGLowIntensity,A
ADCm_end
      }
      break;
   case 's':                           /* save cursor location               */
      asm {
      MOV   ANSISavedRow,CGCurrentRow
      MOV   ANSISavedCol,CGCurrentCol
      }
      break;
   case 'u':                           /* restore cursor location            */
      asm {
      MOV   A,CGNumCharRows
      DEC   A
      CJNE  A,ANSISavedRow,ADCu_0      /* C set if max-1 < saved             */
ADCu_0
      MOV   CGCurrentRow,A             /*  assume min is max-1               */
      JC    ADCu_1
      MOV   CGCurrentRow,ANSISavedRow  /*  wrong, use saved value            */
ADCu_1
      MOV   A,CGNumCharCols
      DEC   A
      CJNE  A,ANSISavedCol,ADCu_2      /* C set if max-1 < saved             */
ADCu_2
      MOV   CGCurrentCol,A             /*  assume min is max-1               */
      JC    ADCu_3
      MOV   CGCurrentCol,ANSISavedCol  /*  wrong, use saved value            */
ADCu_3
      }
      break;
   default:
   }

}

