/* Serial routines for Vid-Link                                              */

#define TRACINGxxx                     /* append xxx to delete tracing!      */
#define ERRTRACExxx                    /* append xxx to stop error tracing   */
#define VIDTESTxxx                     /* append xxx to suppress test code   */

#ifdef TRACING

asm {
TrcMsg   MACRO
         MOV   SBUF,\1
         ACALL TrcWait
         ENDMAC
}

TraceWait() {
asm {
TrcWait
   CLR   IO_TRANSDISABLE
   SETB  IO_TRANSDISABLE
   JNB   TI,TrcWait
   CLR   TI
}
}

#else

asm {
TrcMsg   MACRO
**** nothing here!
         ENDMAC
}

#endif


/*---------------------------------------------------------------------------*/
/* Serial initialization                                                     */

SerInit() {

   asm {

   MOV   SerState,#SER_IDLE            /* start out in idle mode             */
   MOV   SCON,#%01010000               /* mode 1, rec enabled, 8N1, flags 0  */
   ORL   PCON,#%10000000               /* double the bit rate                */
   ANL   TCON,#%00111111               /* flush Timer 1 enable & flag        */
   ANL   TMOD,#$00001111               /* flush Timer 1 controls             */
   ORL   TMOD,#%00100000               /* Timer 1 to Mode 2, 8-bit reload    */
   MOV   TH1,#-6                       /* set up for 9600 b/s                */

   SETB  TR1                           /* start it up                        */
   SETB  IO_TRANSDISABLE               /* disable our net transmitter        */
   SETB  ES                            /* enable serial interrupts           */

   SETB  B_SerTransIdle                /* transmitter is not running now     */
   MOV   NetMsgState,#NMS_IDLE         /* force new line on startup          */
}

}

/*---------------------------------------------------------------------------*/
/* Serial interrupt handler                                                  */
/* There are two pieces:                                                     */
/*  a small chunk at the external interrupt 0 vector in low code space       */
/*  the remainder wherever the linker sees fit to put it                     */

/*---------------------------------------------------------------------------*/
/* Plunk the serial interrupt handler into place                             */
/* We PUSH a few registers to fill up the odd nooks and crannies of EPROM    */
/* The EPROM checksum occupies 2E and 2F, so this is as tight as it gets     */

asm {

SI_here  EQU *                         /* plunk serial vector in place       */

   ORG   $0023

   PUSH  A                             /* save bystanders                    */
   PUSH  B
   PUSH  PSW
   PUSH  ?R0

   LJMP  SerHandler                    /* set serial interrupt vector        */

   ORG   SI_here                       /* return to normal sequence          */

}



/*---------------------------------------------------------------------------*/
/* The main part of the serial interrupt handler goes here                   */
/* Note the bypass controlled by B_SerEcho that handles ordinary echos       */
/*  this should be set only in RS-232 mode to assist manual typing           */
/* We discard LFs and convert CRs into binary zeros to simplify the C code   */

SerHandler(void) {

asm {

/*--- receiver interrupt                                                     */

   JNB   RI,SH_r0                      /* receiver interrupt active?         */
   CLR   RI                            /* yes, clear it                      */

   MOV   A,#SER_RECEIVE                /* are we allowed to receive a char?  */
   CJNE  A,SerState,SH_r0              /* no, so ignore this char            */

   MOV   A,#PUMPFIFOSIZE               /* room for newest char?              */
   CJNE  A,PumpFIFOCtr,SH_r1           /* if not equal to size, yes          */
   SETB  B_SerOverrun                  /* full, set status flag              */
SH_r0
   AJMP  SH_rdone                      /*  ... and ignore the char           */

SH_r1
   MOV   A,SBUF                        /* fetch the character                */
   CJNE  A,#LF,SH_r2a                  /* is it a linefeed?                  */
   SJMP  SH_r0                         /*  yes, throw it away                */
SH_r2a
   MOV   R0,PumpFIFOHead               /* get head pointer                   */
   CJNE  A,#CR,SH_r2                   /* is it a carriage return?           */
   MOV   [R0],#0                       /*  ring gets a binary zero           */
   SJMP  SH_r2b
SH_r2
   MOV   [R0],A                        /* tuck the char into the ring        */
SH_r2b

/* recognize messages aimed at us                                            */

   JB    B_NetUseCS,SH_n0              /* skip net stuff if not in net mode  */
   AJMP  SH_n9a
SH_n0
   PUSH  A                             /* save char around state driver      */

SH_n1
   MOV   A,NetMsgState                 /* networked, figure current state    */
   CJNE  A,#NMS_IDLE,SH_n2             /* at first char of line?             */
   MOV   NetRecLineSum,[R0]            /* yes, initialize the checksum       */
   SETB  B_NetInHeader                 /*  and suppress ring pointer update  */
   TrcMsg #'a'

   CJNE  [R0],#'!',SH_n1a              /* is this a non-checksummed cmd?     */
   MOV   NetMsgState,#NMS_NOCS_BLANK   /*  yes, next char must be blank      */
   TrcMsg #'b'
   SJMP  SH_n9

SH_n1a
   CJNE  [R0],#'#',SH_n2junk           /* if not checksummed, it is junk     */
   MOV   NetMsgState,#NMS_CS_CS1       /*  expect cs, next is first hexit    */
   TrcMsg #'1'
   SJMP  SH_n9

SH_n2
   TrcMsg #'c'
   TrcMsg [R0]
   XCH   A,NetRecLineSum               /* accumulate checksum                */
   TrcMsg A
   ADD   A,[R0]                        /*   CR is zero in ring!              */

   XCH   A,NetRecLineSum
   CJNE  A,#NMS_INVALID,SH_n2a         /* is message invalid right now?      */
   SJMP  SH_n9                         /*  yes, let chars flow through       */

SH_n2a
   CJNE  A,#NMS_NOCS_BLANK,SH_n2b      /* non-cs msg, char must be blank     */
   POP   A                             /*  recover char                      */
   PUSH  A
   CJNE  A,#' ',SH_n2junk              /*  not blank, discard rest of msg    */
   MOV   NetMsgState,#NMS_ACCEPT       /*  blank, accept rest of message     */
   TrcMsg #'d'
   SJMP  SH_n9

SH_n2b
   CJNE  A,#NMS_CS_CS1,SH_n2c          /* cs msg, process most-sig cs hexit  */
   TrcMsg #'2'
   MOV   A,[R0]                        /*  add high cs hexit in high nybble  */
   TrcMsg A
   ACALL HexToNyb
   TrcMsg A
   SWAP  A
SH_n2ba
   ADD   A,NetRecLineSum               /*  combine with existing checksum    */
   CLR   C
   SUBB  A,[R0]                        /*  remove ASCII cs character         */
   ADD   A,#'0'                        /*  replace with ASCII zero           */
   MOV   NetRecLineSum,A
   INC   NetMsgState                   /* step to next state                 */
   SJMP  SH_n9

SH_n2c
   CJNE  A,#NMS_CS_CS2,SH_n2d          /* cs msg, get second cs hexit        */
   TrcMsg #'3'
   MOV   A,[R0]                        /*  add low cs hexit in low nybble    */
   TrcMsg A
   ACALL HexToNyb
   TrcMsg A
   SJMP  SH_n2ba                       /* continue as for high cs hexit      */

/* common endpoints put in the middle of the code to make short JMPs work    */

SH_n2junk
   MOV   NetMsgState,#NMS_INVALID      /* invalid state, discard message     */
SH_n9
   POP   A                             /* recover the input char             */
SH_n9a
   AJMP  SH_n99

SH_n2d
   CJNE  A,#NMS_CS_BLANK1,SH_n2e       /* cs msg, blank after checksum       */
   TrcMsg #'4'
   TrcMsg [R0]

   CJNE  [R0],#' ',SH_n2junk           /*  if so, discard rest of line       */
   INC   NetMsgState                   /*  got blank, expect our name        */
   PUSH  DPH                           /*  set up pointer to name string     */
   PUSH  DPL
   MOV   A,NetNameSel
   RL    A
   ADD   A,#NetNames
   MOV   DPL,A
   MOV   A,#=NetNames
   ADDC  A,#0
   MOV   DPH,A
   CLR   A
   MOVC   A,[A+DPTR]
   MOV   pNetName,A
   INC   DPTR
   CLR   A
   MOVC   A,[A+DPTR]
   MOV   pNetName+1,A
   POP   DPL
   POP   DPH
   SJMP  SH_n9

SH_n2e
   CJNE  A,#NMS_CS_ADDR,SH_n2g         /* match msg addr vs our name         */
   TrcMsg #'5'

   PUSH  DPH
   PUSH  DPL
   MOV   DPL,pNetName
   MOV   DPH,pNetName+1
   CLR   A
   MOVC  A,[A+DPTR]
   TrcMsg A

   INC   DPTR
   MOV   pNetName,DPL
   MOV   pNetName+1,DPH
   POP   DPL
   POP   DPH
   JZ    SH_n2ea                       /* done with node name?               */
   XRL   A,[R0]                        /*  no, char matches our char?        */
   JNZ   SH_n2junk                     /*  no, so discard rest of string     */
   SJMP  SH_n9                         /*  yes, keep on matching             */

SH_n2ea
   TrcMsg #'6'                         /* done with name, match number       */
   MOV   A,[R0]                        /* strip digit to BCD                 */
   TrcMsg A
   ANL   A,#$0F
   CJNE  A,NetNodeNum,SH_n2junk1
   INC   NetMsgState                   /*  got a match, expect second blank  */
   SJMP  SH_n9

SH_n2g
   CJNE  A,#NMS_CS_BLANK2,SH_n2h       /* expect blank after message         */
   TrcMsg #'7'
   TrcMsg [R0]

   CJNE  [R0],#' ',SH_n2junk1          /*  if so, discard rest of line       */
   INC   NetMsgState                   /*  got blank, expect our address     */
   AJMP  SH_n9

SH_n2h
   CJNE  A,#NMS_CS_REST,SH_n2i         /* continue with checksummed cmd      */
   CLR   B_NetInHeader                 /*  so we can store the chars         */
   TrcMsg #'8'
   TrcMsg [R0]
   AJMP  SH_n9

SH_n2i
   CJNE  A,#NMS_ACCEPT,SH_n2junk1      /* command is known good              */
   CLR   B_NetInHeader                 /*  so we can store the chars         */
   TrcMsg #'e'
   AJMP  SH_n9

SH_n2junk1
   AJMP  SH_n2junk

SH_n99

/* update the ring pointers, echo char, and check for end-of-line conditions */
/* raw char in A, cooked char in [R0]                                        */
/* If we are in the header we do NOT advance the pointers!                   */
/*  ... mismatched addresses leave B_NetInHeader set during whole message    */

   JB    B_NetInHeader,SH_r3a          /* do not save header chars           */

   INC   R0                            /* tick the head pointer              */
   CJNE  R0,#PumpFIFO+PUMPFIFOSIZE,SH_r3           /* check for wrap         */
   MOV   R0,#PumpFIFO
SH_r3
   MOV   PumpFIFOHead,R0
   INC   PumpFIFOCtr                   /* tick ring counter                  */
SH_r3a

SH_r4
   JNB   B_SerEcho,SH_r4aa             /* shall we echo it directly?         */
   MOV   SBUF,A                        /* yes, send it out                   */
SH_r4a
   JNB   TI,SH_r4a                     /* and wait for good timing           */
   CLR   TI                            /*  ... but do not allow trans int    */

SH_r4aa
   CJNE  A,#CR,SH_rdone                /* is it a CR?                        */
#ifdef ERRTRACE
   SETB  IO_TRANSDISABLE               /*** flag start of CR processing    ***/
#endif
   JNB   B_NetUseCS,SH_r4OK            /* yes, are we in network mode?       */
   MOV   A,NetMsgState                 /* yes, test current line state       */
   CJNE  A,#NMS_ACCEPT,SH_r4b          /*  is it OK without checksum test?   */
   TrcMsg #'A'
   SJMP  SH_r4OK                       /*   yes, use it                      */
SH_r4b
   CJNE  A,#NMS_CS_REST,SH_r4NG        /*  is it OK with checksum test?      */
   TrcMsg #'B'
   MOV   A,NetRecLineSum               /* yes, is checksum zero now?         */
   TrcMsg A
   JZ    SH_r4OK                       /*  yes, flag line as OK              */

SH_r4NG
   TrcMsg #'X'
#ifdef ERRTRACE
   SETB  IO_TRANSDISABLE               /*** flag start of error processing ***/
#endif
   JBC   B_NetInHeader,SH_r4x          /*  no, skip flush if no chars saved  */

   MOV   PumpFIFOCtr,#0                /*  no, flush FIFO and VRAM buffers   */
   MOV   PumpFIFOHead,#PumpFIFO
   MOV   PumpFIFOTail,#PumpFIFO
   MOV   PumpLine,#PUMP_RECLINE
   MOV   PumpState,#PUMP_INITLINE
   MOV   A,#PUMP_IDLE                  /* wait for VRAM flush                */
#ifdef ERRTRACE
   SETB  IO_TRANSDISABLE               /*** flag start of flushing         ***/
#endif
SH_r4w
   CJNE  A,PumpState,SH_r4w            /* this may delay one full field!     */
   MOV   PumpState,#PUMP_TOVRAM

SH_r4x
   MOV   NetMsgState,#NMS_IDLE         /* prepare for next input line        */
#ifdef ERRTRACE
   SETB  IO_TRANSDISABLE               /*** flag start of error processing ***/
#endif
   SJMP  SH_rdone                      /* and bail out                       */

SH_r4OK
   TrcMsg #'C'
   MOV   SerState,#SER_ENDLINE         /* line is valid, allow processing    */
   MOV   NetMsgState,#NMS_IDLE         /* and reset net message processor    */
#ifdef ERRTRACE
   SETB  IO_TRANSDISABLE               /*** flag start of error processing ***/
#endif

SH_r4c
   MOV   A,#LF                         /*  ... add LF after CR               */
   SJMP  SH_r4                         /*  and send it out if echoing        */

SH_rdone

/*--- transmitter interrupt                                                  */

   JNB   TI,SH_tdone                   /* transmitter interrupt active?      */
   CLR   TI                            /* yes, clear interrupt flag          */

   MOV   A,#SER_TRANSMIT               /* allowed to send another char?      */
   CJNE  A,SerState,SH_tdone           /* no, so ignore this interrupt       */

   MOV   A,PumpFIFOCtr                 /* yes, anything to send?             */
   JNZ   SH_t1                         /* yes, so handle it                  */
   JNB   B_SerUseTE,SH_t2              /* no, using network mode?            */
   MOV   A,#50                         /* yes, wait for full stop bit time   */
SH_t3
   DJNZ  A,SH_t3
   SETB  IO_TRANSDISABLE               /* then disable the transmitter       */
SH_t2
   SETB  B_SerTransIdle                /* indicate we are did a whole ring   */
   MOV   SerState,#SER_IDLE            /* and are now finished               */
   SJMP  SH_tdone

SH_t1
   CLR   B_SerTransIdle                /* indicate we are on the case        */
   JNB   B_SerUseTE,SH_t4              /* using network transmitter?         */
   CLR   IO_TRANSDISABLE               /* yes, so turn on the driver         */
SH_t4
   MOV   R0,PumpFIFOTail               /* fetch char from ring               */
   MOV   SBUF,[R0]                     /*  ... send it                       */

   INC   R0                            /* tick the index                     */
   CJNE  R0,#PumpFIFO+PUMPFIFOSIZE,SH_t5           /* check for wrap         */
   MOV   R0,#PumpFIFO
SH_t5
   MOV   PumpFIFOTail,R0
   DEC   PumpFIFOCtr                   /* tick ring counter                  */

SH_tdone

*--- clean up & go home

SHdone
   POP   ?R0
   POP   PSW
   POP   B
   POP   A
   RETI

*--- convert from hex digit to binary
*    ignores subtle issues like valid hexits, but ignores case correctly

HexToNyb
   CJNE  A,#$3A,HN_1                   /* C set if A < 3A                    */
HN_1
   JC    HN_2                          /* is it numeric?                     */
   ADD   A,#9                          /*  no, coerce 41 -> 4A, 61 -> 6A     */
HN_2
   ANL   A,#$0F                        /* strip off junk                     */
   RET

}

}

/*---------------------------------------------------------------------------*/
/* Flush remaining chars in serial input buffer                              */
/* This simply resets a few key variables without doing any real work        */
/* Call this to discard the input =before= creating the output               */
/*  The stuff in the video buffer is cleared by the next getch()             */

SerFlushInput() {

#ifdef xxx
   putstr("Flushing serial, Pump: ");
   putch('0' + PumpState);
   putstr("\n");
#endif

asm {
   MOV   PumpState,#PUMP_IDLE
   MOV   PumpFIFOCtr,#0
   MOV   PumpFIFOHead,#PumpFIFO
   MOV   PumpFIFOTail,#PumpFIFO
   MOV   SerState,#SER_IDLE
   MOV   NetMsgState,#NMS_IDLE
}


}


/*---------------------------------------------------------------------------*/
/* Wait for the serial transmitter to go idle                                */
/* You call this when you =expect= it to go idle sometime soon...            */

SerWaitOutDone() {

#ifdef xxx
   putstr("\nEnter serial wait, Pump: ");
   putch('0' + PumpState);
   putstr("\n");
#endif

   while (PUMP_IDLE != PumpState) {    /* wait for buffer to empty           */
      continue;
   }

   do {                                /* wait for final character           */
      TESTBIT(TestBits,B_SerTransIdle);
   } while (!TestBits);

#ifdef xxx
   putstr(" ... serial empty\n");
#endif

}

/*---------------------------------------------------------------------------*/
/* Send a character to the serial port via the FIFO and VRAM buffer          */
/* We accumulate a complete line in the VRAM buffer before activating        */
/*  the serial output interrupt handler                                      */

SerPutChar(OutChar)
BYTE OutChar;
{

/*--- first, pop the char into the VRAM buffer                               */
/*    unless it is an EOT, in which case it does not enter the buffer at all */

   if (EOT == OutChar) {
      goto Bypass;
   }

   switch (PumpState) {

   case PUMP_IDLE:
      VidPumpInitLine(PUMP_TRANSLINE,PUMP_INITLINE);        /* set up buffer */

      /* fall through to stick char in FIFO                                  */
   case PUMP_TOVRAM:
      while (PumpFIFOCtr >= PUMPFIFOSIZE) {
         continue;
      }
      TestBits = OutChar;
      asm {
      MOV   R0,PumpFIFOHead
      MOV   [R0],TestBits
      INC   R0
      CJNE  R0,#PumpFIFO+PUMPFIFOSIZE,SPC_1
      MOV   R0,#PumpFIFO
SPC_1
      MOV   PumpFIFOHead,R0
      }
      PumpFIFOCtr++;
      PumpState = PUMP_TOVRAM;
      break;

   case PUMP_FROMVRAM:                 /* problem... pumping the wrong way   */
#ifdef xxx
      SETBIT(B_ConVideo);
      putstr("Wrong way pump in SerPutChar\n");
      CLRBIT(B_ConVideo);
#endif
      PumpState = PUMP_IDLE;
      break;

   default:                            /* oops... ignore the character       */
#ifdef xxx
      SETBIT(B_ConVideo);
      putstr("Hit the default in SerPutChar\n");
      CLRBIT(B_ConVideo);
#endif
      PumpState = PUMP_IDLE;
   }

Bypass:

/*--- if it is a NL or EOT we activate the serial output handler             */

   if ((NL == OutChar) || (EOT == OutChar)) {
      while (PumpFIFOCtr) {                        /* wait for empty FIFO    */
         continue;
      }

      VidPumpInitLine(PUMP_TRANSLINE,PUMP_RESTARTLINE);     /* clear index   */
      PumpState = PUMP_FROMVRAM;                   /* turn buffer around     */
      while ((PUMPFIFOSIZE > PumpFIFOCtr) &&
              (PUMP_FROMVRAM == PumpState)) {
         continue;
      }
      SerState = SER_TRANSMIT;                     /* activate serial output */
      SETBIT(TI);                                  /* kickstart the port     */

      while ((SER_IDLE != SerState) ||
             (PUMP_IDLE != PumpState)) {           /* wait for completion    */
         continue;
      }
   }

}


/*---------------------------------------------------------------------------*/
/* Return 0 if no char available, 1 if any chars remain                      */
/* You call this =only= while processing a line, so chars are available      */
/*  until the last char drains from the FIFO and VRAM buffer                 */
/* The absolute last character is a binary zero, which is convenient...      */

SerCheckChar(void) {

   asm {
      MOV R0,PumpFIFOTail
      MOV TestBits,[R0]
   }

   return ((PUMP_FROMVRAM == PumpState) || (PumpFIFOCtr > 0)) &&
          TestBits;

}


/*---------------------------------------------------------------------------*/
/* Fetch a character from the serial port via the FIFO and VRAM buffer       */
/* The whole line must accumulate in the video buffer before we commence     */
/*   so most of the action occurs on the first entry with an idle port       */
/* If we are in network mode, we discard the header before returning         */

SerGetChar(void) {
BYTE RetChar;

   while (SER_TRANSMIT == SerState) {              /* finish outbound msg    */
      continue;
   }

/*--- dispense chars from FIFO, refilling with new lines as necessary        */

   switch (SerState) {

   case SER_RECEIVE:             /* invalid state, so force a restart        */
      /* fall through to restart serial port                                 */

   case SER_IDLE:
      VidPumpInitLine(PUMP_RECLINE,PUMP_INITLINE);
      PumpState = PUMP_TOVRAM;
      SerState = SER_RECEIVE;

/*    CLRBIT(IO_TRANSDISABLE); */
      while (SER_ENDLINE != SerState) {            /* wait for whole line    */
#ifdef VIDTEST
         if (VidFieldCtr == 1) {
            CLRBIT(B_VidEnable);
         }
         else {
            SETBIT(B_VidEnable);
         }
#endif
         continue;
      }
/*    SETBIT(IO_TRANSDISABLE); */

      while (PumpFIFOCtr) {                        /* wait for empty FIFO    */
         continue;
      }

      VidPumpInitLine(PUMP_RECLINE,PUMP_RESTARTLINE);       /* flip buffer   */
      PumpState = PUMP_FROMVRAM;

      while ((PUMPFIFOSIZE > PumpFIFOCtr) &&       /* wait for FIFO refill   */
              (PUMP_FROMVRAM == PumpState)) {
         continue;
      }

      /* fall through to dispense a character                                */

   case SER_ENDLINE:
      switch (PumpFIFOCtr) {
      case 0:
#ifdef xxx
         SETBIT(B_ConVideo);
         putstr("FIFO empty with ENDLINE set in SerGetChar\n");
         CLRBIT(B_ConVideo);
#endif
         RetChar = 0;
         break;
      case 1:
         SerState = SER_IDLE;
         /* fall through to return the last character                        */
      default:
         asm {
         MOV   R0,PumpFIFOTail         /* fetch char from the FIFO           */
         MOV   TestBits,[R0]
         INC   R0
         CJNE  R0,#PumpFIFO+PUMPFIFOSIZE,SGC_1
         MOV   R0,#PumpFIFO
SGC_1
         MOV   PumpFIFOTail,R0
         DEC   PumpFIFOCtr
         }
         RetChar = TestBits;
      }
   }

   return RetChar;

}

