/* Video.H -- video handlers                                                 */

/*---------------------------------------------------------------------------*/
/* Global constants                                                          */

#define MAX_ACTIVE_LINES   242         /* number of active video lines       */
#define MAX_ACTIVE_DOTS    256         /* number of active bytes/line        */
                                       /* hardware fumbles last two dots     */

#define VID_JUNKLINE       242         /* this line gets all timer updates   */

/*---------------------------------------------------------------------------*/
/* Data pump                                                                 */

/*--- state variables                                                        */
/*    must be even to allow AJMP branches                                    */

#define PUMP_IDLE          0           /* nothing happening                  */
#define PUMP_FROMVRAM      2           /* move data from VRAM to   IRAM      */
#define PUMP_TOVRAM        4           /* move data to   VRAM from IRAM      */
#define PUMP_INITLINE      6           /* initialize a data line             */
#define PUMP_RESTARTLINE   8           /* restart data from a line           */

/*--- location of various data lines                                         */
/*    these can be < MAX_ACTIVE_LINES if CLEAR_NORMAL isn't defined          */
/*    that makes them visible so you can watch data slosh to and fro         */

#define PUMP_RECLINE       243         /* received chars go here             */
#define PUMP_TRANSLINE     244         /* transmitted chars go here          */

/*--- buffer structure                                                       */

struct pump_buffer {
   BYTE PBufferLine[240];              /* the buffer itself                  */
   BYTE PBufferIndex;                  /* offset of current char             */
   BYTE PBufferRemaining;              /* number of chars left in buffer     */
};

#define PB_LINESIZE        250         /* number of bytes in the line        */
#define PB_INDEXLOC        250         /* offset of char index               */
#define PB_REMAINLOC       251         /* offset of length byte              */

/*---------------------------------------------------------------------------*/
/* Timing constants                                                          */

/*--- duration of various pulses measured in I8254 ticks (500 ns)            */

#define PERIOD_NORMAL      127         /* normal line period                 */
#define PERIOD_HALFSHORT   63          /* halfline short by one clock        */
#define PERIOD_HALFLONG    64          /* halfline long by one clock         */

#define SYNC_NORMAL        10          /* normal sync pulse duration         */
#define SYNC_EQUALIZING    5           /* equalizing pulse duration          */
#define SYNC_VSYNC         54          /* vertical sync pulse duration       */

#define BLANK_NORMAL       23          /* normal blanking                    */
#define BLANK_HALFLINE     60          /* blank for half-lines               */
#define BLANK_COMPLETE     255         /* blank for entire line              */

/*--- duration of delays between interesting events                          */
/*    measured in Timer 0 ticks of 1.085 us                                  */
/*    remember to negate these before loading them into the timer registers  */
/*     (XASM cannot handle signed number in arithmetic operations...)        */

#define T0_NORMLINE     58             /* normal video line                  */
#define T0_HALFLINE     29             /* half video line                    */
#define T0_QUARTERLINE  14             /* half video line                    */
#define T0_ACTIVE       14045          /* 240 full lines                     */

/*---------------------------------------------------------------------------*/
/* Video state handlers                                                      */
/* See the branch table in VIDSYNC.C to make sure these are correct...       */
/*  the numbers must match up with the location in the branch table          */

#define VS_IDLE            0           /* nothing happens here               */
#define VS_VSYNC           1           /* start vertical sync pulse          */
#define VS_EQ_SHORT        2           /* set equalizing pulses 2 count short*/
#define VS_F1_BLANK        3           /* field 1 vertical blanking          */
#define VS_F1_ACTIVE       4           /* field 1 start of active video      */
#define VS_F1_END          5           /* field 1 last active line           */
#define VS_F2_BLANK        6           /* field 2 vertical blanking          */
#define VS_F2_ACTIVE       7           /* field 2 start of active video      */
#define VS_F2_END          8           /* field 2 last active line           */
#define VS_DATAPUMP        9           /* pump data between IRAM and VRAM    */


